/*
                  Restart Configuration File: Load & Save

	Functions:

	int RCLoadFromFile(char *filename)
	int RCSaveToFile(char *filename)

	---



 */

#include "keymap.h"
#include "univlist.h"

#include "xsw.h"


#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))


/*
 *	Macro to parse standard color string and set
 *	values into WColorStruct c.
 */
int RC_SET_COLOR(
	char *string,
	WColorStruct *c
)
{
	if((string == NULL) ||
	   (c == NULL)
	)
	    return(-1);

	c->a = 0x00;

	return(
	    StringParseStdColor(
                string,
                &(c->r),
	        &(c->g),
	        &(c->b)
            )
	);
}



int RCLoadFromFile(char *filename)
{
	int i, status;
        char *strptr, *strptr2, *strptr3;

        FILE *fp;
        off_t filesize;
        struct stat stat_buf;

        char parm[CFG_PARAMETER_MAX];
        char val[CFG_VALUE_MAX];
        int lines_read = 0;

	char cwd[PATH_MAX];

	int entry_num;


	/* Check if filename exists. */
	if(filename == NULL)
	    return(-1);
	if(stat(filename, &stat_buf))
	{
	    fprintf(stderr, "%s: No such file.\n", filename);
	    return(-1);
	}

	/* Get size of file. */
        filesize = stat_buf.st_size;

        /* Open filename. */
        fp = fopen(filename, "r");
        if(fp == NULL)
        {
            fprintf(stderr, "%s: Cannot open.\n", filename);
            return(-1);
        }

	/* ************************************************************* */

	/* Free all universe entries. */
	UnivDeleteAll();

	/* Free all quick menu entries. */
	QMenuFreeAllEntries();


	/* ******************************************************** */

	strptr = NULL;

        while(1)
        {
	    /* Free previous line and allocate/read next line. */
            free(strptr); strptr = NULL;
            strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
            if(strptr == NULL) break;
            lines_read++;

            /* Fetch parameter. */
            strptr2 = StringCfgParseParm(strptr);
            if(strptr2 == NULL) continue;
            strncpy(parm, strptr2, CFG_PARAMETER_MAX);
            parm[CFG_PARAMETER_MAX - 1] = '\0';

            /* Fetch value. */
            strptr2 = StringCfgParseValue(strptr);
            if(strptr2 == NULL) strptr2 = "0";  /* Set it to "0" if NULL. */
            strncpy(val, strptr2, CFG_VALUE_MAX);
            val[CFG_VALUE_MAX - 1] = '\0';


	    /* VersionMajor */
            if(!strcmp(parm, "VersionMajor"))
            {
		option.rc_version_major = atoi(val);
	    }
            /* VersionMinor */
            else if(!strcmp(parm, "VersionMinor"))
            {
                option.rc_version_minor = atoi(val);
            }

	    /* ToplevelDir */
            else if(!strcmp(parm, "ToplevelDir"))
            {
		strptr3 = PathSubHome(val);
		if(strptr3 != NULL)
		{
		    strncpy(val, strptr3, CFG_VALUE_MAX);
		    val[CFG_VALUE_MAX - 1] = '\0';
		}

		if(ISPATHABSOLUTE(val))
		{
		    strncpy(dname.toplevel, val, PATH_MAX);
		}
		else
		{
		    strncpy(
			dname.toplevel,
			PrefixPaths(getcwd(cwd, PATH_MAX), val),
			PATH_MAX
		    );
		}
		dname.toplevel[PATH_MAX - 1] = '\0';

		if(!ISPATHDIR(dname.toplevel))
		{
		    fprintf(stderr,
			"%s: Line %i: %s: Cannot access directory.\n",
			filename, (int)lines_read, dname.toplevel
		    );
		}
	    }
	    /* ImagesDir */
            else if(!strcmp(parm, "ImagesDir"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {   
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

		if(ISPATHABSOLUTE(val))
		{
		    strncpy(dname.images, val, PATH_MAX);
		}
		else
		{
                    strncpy(
                        dname.images,
                        PrefixPaths(dname.toplevel, val),
                        PATH_MAX
                    );
		}
		dname.images[PATH_MAX - 1] = '\0';

                if(!ISPATHDIR(dname.images))
                {
                    fprintf(stderr,
                        "%s: Line %i: %s: Cannot access directory.\n",
                        filename, (int)lines_read, dname.images
                    );
                }
            }
	    /* SoundsDir */
            else if(!strcmp(parm, "SoundsDir"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {   
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(ISPATHABSOLUTE(val))
                {
                    strncpy(dname.sounds, val, PATH_MAX);
                }
		else
                {
                    strncpy(
                        dname.sounds,
                        PrefixPaths(dname.toplevel, val),
                        PATH_MAX
                    );
                }
		dname.sounds[PATH_MAX - 1] = '\0';

                if(!ISPATHDIR(dname.sounds))
                {
                    fprintf(stderr,
                        "%s: Line %i: %s: Cannot access directory.\n",
                        filename, (int)lines_read, dname.sounds
                    );
                }
            }
            /* EtcDir */
            else if(!strcmp(parm, "EtcDir"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {   
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(ISPATHABSOLUTE(val))
                {
                    strncpy(dname.etc, val, PATH_MAX);
                }
                else
                {
                    strncpy(
                        dname.etc,
                        PrefixPaths(dname.toplevel, val),
                        PATH_MAX
                    );
                }
		dname.etc[PATH_MAX - 1] = '\0';

                if(!ISPATHDIR(dname.etc))
                {
                    fprintf(stderr,
                        "%s: Line %i: %s: Cannot access directory.\n",
                        filename, (int)lines_read, dname.etc
                    );
                }   
            }

	    /* NetDeviceMaxLoad */
	    else if(!strcmp(parm, "NetDeviceMaxLoad"))
	    {
		loadstat.net_load_max = atol(val);
                if(loadstat.net_load_max < 800)
                {
                    fprintf(stderr,
    "%s: Line %i: Error: Max net device load setting %ld, is too low.\n",
                            filename,
                            lines_read,
                            loadstat.net_load_max
                    );
		    /* Set to 800. */
		    loadstat.net_load_max = 800;
                }
	    }
	    /* NetIntervalAutoTune */
	    else if(!strcmp(parm, "NetIntervalAutoTune"))
	    {
		auto_interval_tune.state = StringIsYes(val);
	    }
            /* NetIntervalTuneInterval */
            else if(!strcmp(parm, "NetIntervalTuneInterval"))
            {
		auto_interval_tune.interval = atol(val);
		if(auto_interval_tune.interval < 1000)
		{
                    fprintf(stderr,
   "%s: Line %i: Warning: Auto net interval tune interval %i ms, is very short.\n",
                        filename,
                        (int)lines_read,
                        (int)auto_interval_tune.interval
                    );
		}
            }
            /* DefaultServerAddress */
            else if(!strcmp(parm, "DefaultServerAddress"))
            {
		if(net_parms.is_address_set == 0)
		{
		    strncpy(net_parms.address, val,
			MAX_URL_LEN);
		}
		net_parms.address[MAX_URL_LEN - 1] = '\0';
            }
            /* DefaultServerPort */
            else if(!strcmp(parm, "DefaultServerPort"))
            {
                if(net_parms.is_address_set == 0)
                {
		    net_parms.port = atoi(val);
		}
	        if(net_parms.port < 0)
		    net_parms.port = 0;
            }
            /* DefaultLoginName */
            else if(!strcmp(parm, "DefaultLoginName"))
            {
		if(net_parms.is_address_set == 0)
		{
                    strncpy(net_parms.login_name, val, XSW_OBJ_NAME_MAX);
                    net_parms.login_name[XSW_OBJ_NAME_MAX - 1] = '\0';

		    switch(DBValidateObjectName(net_parms.login_name))
		    {
		      case -1:
		        fprintf(stderr,
			    "%s: Invalid name.\n",
		            net_parms.login_name
		        );
		        break;

		      case 1:
                        fprintf(stderr,
                            "%s: Name too long.\n",
                            net_parms.login_name
                        );
		        break;

                      case 2:
                        fprintf(stderr,
                            "%s: Name too short.\n",
                            net_parms.login_name
                        );
		        break;

                      case 3:
                        fprintf(stderr,
                            "%s: Name contains invalid character(s).\n",
                            net_parms.login_name
                        );
                        break;

		      default:
		        break;
		    }
		}
            }
            /* DefaultLoginPassword */
            else if(!strcmp(parm, "DefaultLoginPassword"))
            {
                if(net_parms.is_address_set == 0)
                {
                    strncpy(
			net_parms.login_password,
			val,
		        XSW_OBJ_PASSWORD_MAX
		    );
                    net_parms.login_password[XSW_OBJ_PASSWORD_MAX - 1] = '\0';
		}
            }
	    /* ThrottleScopeMode */
	    else if(!strcmp(parm, "ThrottleScopeMode"))
	    {
		if(strpfx(val, "B") ||
                   strpfx(val, "b")
		)
		{
		    option.throttle_mode = THROTTLE_MODE_BIDIRECTIONAL;
		}
                else if(strpfx(val, "N") ||
                        strpfx(val, "n")
		)
                {
                    option.throttle_mode = THROTTLE_MODE_NORMAL;
                }
		else
		{
		    option.throttle_mode = THROTTLE_MODE_NORMAL;
		}
	    }
            /* Local updates. */
            else if(!strcmp(parm, "LocalUpdates"))
            {
                option.local_updates = StringIsYes(val);
            }
            /* AutoMapEconomyWindow */
            else if(!strcmp(parm, "AutoMapEconomyWindow"))
            {
                option.auto_map_eco_win = StringIsYes(val);
            }
            /* AutoMapUniverseListWindow */
            else if(!strcmp(parm, "AutoMapUniverseListWindow"))
            {
                option.auto_map_univ_list_win = StringIsYes(val);
            }
	    /* AutoViewscreenZoom */
            else if(!strcmp(parm, "AutoViewscreenZoom"))
	    {
		option.auto_zoom = StringIsYes(val);
            }
            /* ViewScreenMarks (was DisplayHeadingArrow) */
            else if(!strcmp(parm, "ViewScreenMarks") ||
                    !strcmp(parm, "DisplayHeadingArrow")
	    )
            {
		option.show_viewscreen_marks = StringIsYes(val);
            }
            /* ViewScreenLabels */
            else if(!strcmp(parm, "ViewScreenLabels"))
            {
		option.show_viewscreen_labels = atoi(val);

		if(option.show_viewscreen_labels < 0)
		    option.show_viewscreen_labels = 0;
                else if(option.show_viewscreen_labels > 3)
		    option.show_viewscreen_labels = 3;
            }
	    /* ShowLensFlares */
            else if(!strcmp(parm, "ShowLensFlares"))
            {
                option.show_lens_flares = StringIsYes(val);
            }
            /* ShowStrobeGlow */
            else if(!strcmp(parm, "ShowStrobeGlow"))
            {
                option.show_strobe_glow = StringIsYes(val);
            }
            /* ShowNebulaGlow */
            else if(!strcmp(parm, "ShowNebulaGlow"))
            {
                option.show_nebula_glow = StringIsYes(val);
            }
	    /* FormalNameLabels */
	    else if(!strcmp(parm, "FormalNameLabels"))
	    {
		option.show_formal_label = atoi(val);

                if(option.show_formal_label < 0)
                    option.show_formal_label = 0;
                else if(option.show_formal_label > 2)
                    option.show_formal_label = 2;
	    }
	    /* NotifyScannerContacts */
	    else if(!strcmp(parm, "NotifyScannerContacts"))
            {
                option.notify_scanner_contacts = StringIsYes(val);
            }
	    /* ShowNetErrors */
            else if(!strcmp(parm, "ShowNetErrors"))
            {
                option.show_net_errors = StringIsYes(val);
            }
            /* ShowServerErrors */
            else if(!strcmp(parm, "ShowServerErrors"))
            {
                option.show_server_errors = StringIsYes(val);
            }
	    /* Sounds */
            else if(!strcmp(parm, "Sounds"))
            {
		option.sounds = atoi(val);

		if(option.sounds < 0)
		    option.sounds = 0;
	    }
	    /* ControlType */
            else if(!strcmp(parm, "ControlType"))
            {
		if(!option.cmd_line_set_controller)
		{
		    /* Pointer. */
                    if(strcasepfx(val, "p"))
                    {
                        option.controller = CONTROLLER_POINTER;
                    }
		    /* Keyboard. */
                    else if(strcasepfx(val, "k"))
                    {
                        option.controller = CONTROLLER_KEYBOARD;
                    }
#ifdef JS_SUPPORT
		    /* Joystick. */
                    else if(strcasepfx(val, "j"))
		    {
		        option.controller = CONTROLLER_JOYSTICK;
		    }
#endif /* JS_SUPPORT */
		    else
		    {
		        fprintf(stderr,
                            "%s: Line %i: ControlType: Bad value `%s'.\n",
                            filename,
                            lines_read,
                            val
                        );
		    }
                    /*   Change over to and initialize game controller.
                     *   Global option.controller will be re-updated by this
                     *   call.
                     */
                    GCtlInit(option.controller);
		}
            }
	    /* Units */
            else if(!strcmp(parm, "Units"))
            {
		if(strcasepfx(val, "XSW"))
		    option.units = XSW_UNITS_XSW;
		else if(strcasepfx(val, "AstroMetric") ||
                        strcasepfx(val, "Metric")
		)
                    option.units = XSW_UNITS_METRIC;
                else if(strcasepfx(val, "English"))
                    option.units = XSW_UNITS_ENGLISH;
		else
		    fprintf(stderr,
 "%s: Line %i: Warning: Unsupported units `%s'.\n",
                        filename,
                        lines_read,
			val
                    );
	    }
	    /* AsyncImageLoading */
	    else if(!strcmp(parm, "AsyncImageLoading"))
            {
		option.async_image_loading = StringIsYes(val);
	    }
            /* AsyncImageLoadPixels */
            else if(!strcmp(parm, "AsyncImageLoadPixels"))
            {
		option.async_image_pixels = atoi(val);
            } 
	    /* SoundServerStartCommand */
	    else if(!strcmp(parm, "SoundServerStartCommand"))
	    {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

		if(!ISPATHABSOLUTE(val))
		{
		    fprintf(stderr,
 "%s: Line %i: Error: Sound server location must be an absolute path.\n",
			filename,
			lines_read
		    );
                    continue;
		}
		strncpy(sound.start_cmd, val, PATH_MAX + NAME_MAX);
		sound.start_cmd[PATH_MAX + NAME_MAX - 1] = '\0';

                if(stat(sound.start_cmd, &stat_buf) &&
                   (option.sounds > XSW_SOUNDS_NONE)
		)
                {
                    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
                        filename,
                        lines_read,
                        sound.start_cmd
                    );
                }
	    }

#ifdef JS_SUPPORT
            /* JSCaliberationFile */
            else if(!strcmp(parm, "JSCaliberationFile"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

		if(!ISPATHABSOLUTE(val))
		{
		    if(getenv("HOME") != NULL)
		    {
			strncpy(
			    fname.js_calib,
			    PrefixPaths(getenv("HOME"), val),
			    PATH_MAX + NAME_MAX
			);
		    }
		    else
		    {
			strncpy(
                            fname.js_calib,
                            PrefixPaths("/", val),
                            PATH_MAX + NAME_MAX
                        );
		    }
		}
		else
		{
		    strncpy(fname.js_calib, val, NAME_MAX + PATH_MAX);
		}
		fname.js_calib[NAME_MAX + PATH_MAX - 1] = '\0';

		if(stat(fname.js_calib, &stat_buf))
		{
		    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
			filename,
			lines_read,
			fname.js_calib
		    );
		}
            }
#endif /* JS_SUPPORT */
	    /* MainMenuConfFile */
	    else if(!strcmp(parm, "MainMenuConfFile") ||
                    !strcmp(parm, "MainMenuConfigurationFile")
            )
	    {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(!ISPATHABSOLUTE(val))
                {
                    strncpy(
                        fname.main_menu_conf,
                        PrefixPaths(dname.etc, val),
                        PATH_MAX + NAME_MAX  
                    );
                }
                else
                {
                    strncpy(fname.main_menu_conf, val, NAME_MAX + PATH_MAX);
                }
                fname.main_menu_conf[NAME_MAX + PATH_MAX - 1] = '\0';
                if(stat(fname.main_menu_conf, &stat_buf))
                {
                    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
                        filename,
                        lines_read,
                        fname.main_menu_conf
                    );
                }
	    }
	    /* OCSNamesFile */
	    else if(!strcmp(parm, "OCSNamesFile"))
	    {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(!ISPATHABSOLUTE(val))
                {
		    strncpy(
			fname.ocsn,
			PrefixPaths(dname.etc, val),
			PATH_MAX + NAME_MAX
                    );
                }   
                else
                {
                    strncpy(fname.ocsn, val, NAME_MAX + PATH_MAX);
                }
		fname.ocsn[NAME_MAX + PATH_MAX - 1] = '\0';
                if(stat(fname.ocsn, &stat_buf))
                {
                    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
                        filename,
                        lines_read,
                        fname.ocsn
                    );
                }
            }
            /* ISRFile */
            else if(!strcmp(parm, "ISRFile"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(!ISPATHABSOLUTE(val))
                {
                    strncpy(
                        fname.isr,
                        PrefixPaths(dname.images, val),
                        PATH_MAX + NAME_MAX
                    );
                }
                else
                {   
                    strncpy(fname.isr, val, NAME_MAX + PATH_MAX);
                }
		fname.isr[NAME_MAX + PATH_MAX - 1] = '\0';

                if(stat(fname.isr, &stat_buf))
                {
                    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
                        filename,
                        lines_read,
                        fname.isr
                    );
                }
            }
            /* SoundSchemeFile */
            else if(!strcmp(parm, "SoundSchemeFile"))
            {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(!ISPATHABSOLUTE(val))
                {
                    strncpy(
                        fname.sound_scheme,
                        PrefixPaths(dname.sounds, val),
                        PATH_MAX + NAME_MAX
                    );
                }
                else  
                {
                    strncpy(fname.sound_scheme, val, NAME_MAX + PATH_MAX);
                }
		fname.sound_scheme[NAME_MAX + PATH_MAX - 1] = '\0';

                if(stat(fname.sound_scheme, &stat_buf))
                {
                    fprintf(stderr,
                        "%s: Line %i: Warning: %s: No such file.\n",
                        filename,
                        lines_read,
                        fname.sound_scheme
                    );
                }
            }
            /* SoundServerConnectionArgument */
            else if(!strcmp(parm, "SoundServerConnectionArgument"))
            {
		strncpy(sound.con_arg, val, MAX_URL_LEN);
		sound.con_arg[MAX_URL_LEN - 1] = '\0';
            }
	    /* LogFile */
	    else if(!strcmp(parm, "LogFile"))
	    {
                strptr3 = PathSubHome(val);
                if(strptr3 != NULL)
                {
                    strncpy(val, strptr3, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';
                }

                if(!ISPATHABSOLUTE(val))
                {
		    if(getcwd(cwd, PATH_MAX) != NULL)
		    {
                        strncpy(
                            fname.log,
                            PrefixPaths(cwd, val),
                            PATH_MAX + NAME_MAX
                        );
		    }
                }
                else
                {  
                    strncpy(fname.log, val, PATH_MAX + NAME_MAX);
                }
                fname.log[NAME_MAX + PATH_MAX - 1] = '\0';
	    }

	    /* *************************************************** */

            /* ColorStandardText */
            else if(!strcmp(parm, "ColorStandardText"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.standard_text_cv
                );
            }
            /* ColorBoldText */
            else if(!strcmp(parm, "ColorBoldText")) 
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bold_text_cv
                );
            }
            /* ColorWithdrawnText */
            else if(!strcmp(parm, "ColorWithdrawnText"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.withdrawn_text_cv
                );
            }


            /* ColorBPStandardText */
            else if(!strcmp(parm, "ColorBPStandardText"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_standard_text_cv
                );
            }
            /* ColorBPBoldText */
            else if(!strcmp(parm, "ColorBPBoldText"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_bold_text_cv
                );  
            }
            /* ColorBPWithdrawnText */
            else if(!strcmp(parm, "ColorBPWithdrawnText"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_withdrawn_text_cv
                );
            }
            /* ColorBPLightOutline */
            else if(!strcmp(parm, "ColorBPLightOutline"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_light_outline_cv
                );  
            }
            /* ColorBPNormalOutline */
            else if(!strcmp(parm, "ColorBPNormalOutline"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_normal_outline_cv
                );
            }
            /* ColorBPDarkOutline */
            else if(!strcmp(parm, "ColorBPDarkOutline"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_dark_outline_cv
                );  
            }


            /* ColorBPWarning */
            else if(!strcmp(parm, "ColorBPWarning"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_warning_cv
                );  
            }
            /* ColorBPDanger */
            else if(!strcmp(parm, "ColorBPDanger"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_danger_cv
                );
            }
            /* ColorBPCritical */
            else if(!strcmp(parm, "ColorBPCritical"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bp_critical_cv
                );
            }


            /* ColorBPHull */
            else if(!strcmp(parm, "ColorBPHull"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_hull_cv
                );  
            }
            /* ColorBPPower */
            else if(!strcmp(parm, "ColorBPPower"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_power_cv
                );
            }
            /* ColorBPVis */
            else if(!strcmp(parm, "ColorBPVis"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_vis_cv
                );
            }
            /* ColorBPShields */
            else if(!strcmp(parm, "ColorBPShields"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_shields_cv
                );
            }
            /* ColorBPDmgCtl */
            else if(!strcmp(parm, "ColorBPDmgCtl"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_dmgctl_cv
                );
            }
            /* ColorBPThrottle */
            else if(!strcmp(parm, "ColorBPThrottle"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_throttle_cv
                );
            }
            /* ColorBPThrottleRev */
            else if(!strcmp(parm, "ColorBPThrottleRev"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.bpol_throttle_rev_cv
                );
            }

	    /* ColorVSLabelBg */
            else if(!strcmp(parm, "ColorVSLabelBg"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.vs_label_bg_cv
                );
            }
	    /* ColorVSLabelFg */
            else if(!strcmp(parm, "ColorVSLabelFg"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.vs_label_fg_cv
                );
            }

            /* ColorScMarkUnknown */
            else if(!strcmp(parm, "ColorScMarkUnknown"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.scmark_unknown_cv
                );
            }
            /* ColorScMarkLocked */
            else if(!strcmp(parm, "ColorScMarkLocked"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.scmark_locked_cv
                );
            }
            /* ColorScMarkWeapon */
            else if(!strcmp(parm, "ColorScMarkWeapon"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.scmark_weapon_cv
                );
            }
            /* ColorScMarkHome */
            else if(!strcmp(parm, "ColorScMarkHome"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.scmark_home_cv
                );
            }
            /* ColorScMarkArea */
            else if(!strcmp(parm, "ColorScMarkArea"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.scmark_area_cv
                );
            }


            /* ColorKeymapQueryBg */
            else if(!strcmp(parm, "ColorKeymapQueryBg"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.keymap_query_bg_cv
                );
            }
            /* ColorKeymapQueryFg */
            else if(!strcmp(parm, "ColorKeymapQueryFg"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.keymap_query_fg_cv
                );
            }


	    /* ColorHeadingArrow */
            else if(!strcmp(parm, "ColorHeadingArrow"))
	    {
		RC_SET_COLOR(
		    val,
		    &xsw_color.heading_arrow
		);
	    }
            /* ColorLockArrow */
            else if(!strcmp(parm, "ColorLockArrow"))
            {
		RC_SET_COLOR(
                    val,
                    &xsw_color.lock_arrow
		);
            }
            /* ColorLockCursor */
            else if(!strcmp(parm, "ColorLockCursor"))
            {
                RC_SET_COLOR(
		    val,
		    &xsw_color.lock_cursor
                );
            }
            /* ColorStreamGreen */
            else if(!strcmp(parm, "ColorStreamGreen"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.stream_green
                );
            }
            /* ColorStreamOrange */
            else if(!strcmp(parm, "ColorStreamOrange"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.stream_orange
		);
            }
            /* ColorStreamPurple */
            else if(!strcmp(parm, "ColorStreamPurple"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.stream_purple
                );
            }
            /* ColorStreamYellow */
            else if(!strcmp(parm, "ColorStreamYellow"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.stream_yellow
                );
            }
            /* ColorShieldBlue */
            else if(!strcmp(parm, "ColorShieldBlue"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.shield_blue
                );
            }
            /* ColorStarGlow */
            else if(!strcmp(parm, "ColorStarGlow"))
            {
                RC_SET_COLOR(
                    val,
                    &xsw_color.star_glow
                );
            }

            /* **************************************************** */
	    /* BeginKeyMap */
            else if(!strcmp(parm, "BeginKeyMap"))
            {
		/* Begin reading keymap entries. */
                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;

                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';

                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';


		    /* Match keymap parm by conical name. */
		    for(i = 0, status = 0; i < TOTAL_XSW_KEYMAPS; i++)
		    {
			if(!strcasecmp(parm, xsw_keymap_name[i]))
			{
			    xsw_keymap[i].keycode = atoi(val);
			    status = 1;
			}
		    }
		    if(status)
			continue;

                    /* EndKeyMap */
                    if(!strcmp(parm, "EndKeyMap"))
                    {
                        break;
                    }
                    /* Unknown parameter. */
                    else
                    {
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                    }
		}
	    }
            /* **************************************************** */
	    /* BeginSWAddressEntry */
	    /* BeginUniverseEntry */
            else if(!strcmp(parm, "BeginSWAddressEntry") ||
                    !strcmp(parm, "BeginUniverseEntry")
	    )
            {
		/*   Allocate a new universe entry (append).
		 *   Do not pass any fields, we will set them ourselves.
		 */
		status = UnivAdd(
		    NULL, NULL, 0, NULL, -1
		);
		if(status < 0)
		    continue;

		/* Get new entry_num. */
		entry_num = total_univ_entries - 1;
		if(!UnivIsAllocated(entry_num))
		    continue;


		/* Begin loading. */
                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;
                    
                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';
                        
                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);  
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';


                    /* Alias */
                    if(!strcmp(parm, "Alias"))
                    {
                        univ_entry[entry_num]->alias = (char *)
                            calloc(1, (strlen(val) + 1) * sizeof(char));
                        if(univ_entry[entry_num]->alias == NULL)
                        {
                            fprintf(stderr,
                                "%s: Line %i: Memory allocation error.\n",
                                filename, lines_read
                            );
                            continue;
                        }
                        strcpy(
                            univ_entry[entry_num]->alias,
                            val
                        );
		    }
                    /* URL */
                    else if(!strcmp(parm, "URL"))
                    {
                        univ_entry[entry_num]->url = (char *)
                            calloc(1, (strlen(val) + 1) * sizeof(char));
                        if(univ_entry[entry_num]->url == NULL)
                        {
                            fprintf(stderr,
                                "%s: Line %i: Memory allocation error.\n",
                                filename, lines_read
                            );
			    continue;
                        }
			if(strlen(val) > MAX_URL_LEN)
			{
                            fprintf(stderr,
                                "%s: Line %i: Warning: URL exceeds MAX_URL_LEN.\n",
                                filename, lines_read
                            );
			}
                        strcpy(
                            univ_entry[entry_num]->url,
                            val
                        );
                    }
                    /* LastConnected */
                    else if(!strcmp(parm, "LastConnected"))
                    {
                        univ_entry[entry_num]->last_connected = atol(val);
                    }
                    /* Comments */
                    else if(!strcmp(parm, "Comments"))
                    {
                        univ_entry[entry_num]->comments = (char *)
			    calloc(1, (strlen(val) + 1) * sizeof(char));
			if(univ_entry[entry_num]->comments == NULL)
			{
			    fprintf(stderr,
				"%s: Line %i: Memory allocation error.\n",
				filename, lines_read
			    );
                            continue;
			}
			strcpy(
			    univ_entry[entry_num]->comments,
			    val
			);
                    }

                    /* EndSWAddressEntry */
		    /* EndUniverseEntry */
                    else if(!strcmp(parm, "EndSWAddressEntry") ||
                            !strcmp(parm, "EndUniverseEntry")
		    )
                    {
			if((univ_entry[entry_num]->alias == NULL) ||
			   (univ_entry[entry_num]->url == NULL)
			)
			{
                            fprintf(stderr,
                     "%s: Line %i: Warning: Incomplete universe entry #%i.\n",
                                filename,
                                lines_read,
                                entry_num
                            );
			}

                        break;
                    }

                    /* Unknown parameter. */
                    else
                    { 
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    }

                }
            }
	    /* ******************************************************** */
	    /* BeginQuickMenuItem */
            else if(!strcmp(parm, "BeginQuickMenuItem"))
            {
                /* Increment global variable total_qmenu_entries. */
                total_qmenu_entries++;

		/* Reallocate menu item pointer. */
		qmenu_entry = (qmenu_entry_struct **)
		    realloc(qmenu_entry, sizeof(qmenu_entry_struct *)
                        * total_qmenu_entries
		    );
		if(qmenu_entry == NULL)
		{
		    fprintf(stderr,
			"LoadRCFromFile(): Error: Cannot realloc() qmenu entries.\n"
		    );
                    total_qmenu_entries = 0;
		    continue;
		}
		/* Allocate a new entry. */
		qmenu_entry[total_qmenu_entries - 1] = (qmenu_entry_struct *) 
		    calloc(1, sizeof(qmenu_entry_struct));
                if(qmenu_entry[total_qmenu_entries - 1] == NULL)
                {
                    fprintf(stderr,
                        "LoadRCFromFile(): Error: Cannot realloc() qmenu entry %i.\n",
			total_qmenu_entries - 1
                    );
                    total_qmenu_entries--;  
                    continue;
                }
                    
                while(1)
                {
                    /* Free previous line and allocate/read next line. */
                    free(strptr); strptr = NULL;
                    strptr = FReadNextLineAlloc(fp, UNIXCFG_COMMENT_CHAR);
                    if(strptr == NULL) break;
                    lines_read++;

                    /* Fetch parameter. */
                    strptr2 = StringCfgParseParm(strptr);
                    if(strptr2 == NULL) continue;
                    strncpy(parm, strptr2, CFG_PARAMETER_MAX);
                    parm[CFG_PARAMETER_MAX - 1] = '\0';

                    /* Fetch value. */
                    strptr2 = StringCfgParseValue(strptr);   
                    if(strptr2 == NULL) strptr2 = "0";
                    strncpy(val, strptr2, CFG_VALUE_MAX);
                    val[CFG_VALUE_MAX - 1] = '\0';


                    /* Name */
                    if(!strcmp(parm, "Name"))
                    {
		        qmenu_entry[total_qmenu_entries - 1]->name = (char *)
			    calloc(1, (strlen(val) + 1) * sizeof(char));
                        strncpy(
			    qmenu_entry[total_qmenu_entries - 1]->name,
			    val,
			    strlen(val)
			);
                    }
                    /* Type */
                    else if(!strcmp(parm, "Type"))
                    {
			qmenu_entry[total_qmenu_entries - 1]->type = atoi(val);
                    }
		    /* ActionCode */
                    else if(!strcmp(parm, "ActionCode"))
                    {
                        qmenu_entry[total_qmenu_entries - 1]->op_code =
			    atol(val);
                    }

                    /* EndQuickMenuItem */
                    else if(!strcmp(parm, "EndQuickMenuItem"))
                    {
			/* Make sure this quick menu entry is valid. */
			if(qmenu_entry[total_qmenu_entries - 1]->name == NULL)
			{
                            fprintf(stderr,
                    "%s: Line %i: Warning: Incomplete quick menu entry #%i\n",
                                filename,
                                lines_read,
                                total_qmenu_entries - 1
                            );
			}
                        break;
                    }

                    /* Unknown parameter. */
                    else
                    {
                        fprintf(stderr,
                            "%s: Line %i: Unknown parameter: %s\n",
                            filename,
                            lines_read,
                            parm
                        );
                        continue;
                    }
                }
            }
            /* Unknown parameter. */
            else
            {
                fprintf(stderr, "%s: Line %i: Unknown parameter: %s\n",
                    filename,
                    lines_read,
                    parm
                );
            }
	}


	/* Close the file. */
	fclose(fp); fp = NULL;



	/*   Warn if configuration being used is older than version of
	 *   this program.
	 */
	if(option.rc_version_major < XSW_VERSION_MAJOR)
	    status = -1;
	else if(option.rc_version_minor < XSW_VERSION_MINOR)
	    status = -1;
	else
	    status = 0;

	if(status != 0)
	{
	    fprintf(stderr,
"%s: Warning: File version %i.%i is out of date with program version %i.%i.\n",
		filename,
		option.rc_version_major,
		option.rc_version_minor,
		XSW_VERSION_MAJOR,
		XSW_VERSION_MINOR
	    );

	    fprintf(stderr,
"\n\
An older configuration file has been detected, the contents of the file\n\
may be out of date and certain features may be unspecified.\n\
The program can continue running, however you are strongly encuraged to\n\
use an up to date configuration file.\n"
	    );
	}



	return(0);
}


/*
 *	Prints `parameter = value' string by the given values
 *	in label and WColorStruct c.
 */
void RC_PRINT_COLOR_STRING(
	char *string,
	char *label,
	WColorStruct c
)
{
	if(string == NULL)
	    return;
	if(label == NULL)
	    return;

	sprintf(string, "%s = #%.2x%.2x%.2x\n",
	    label,
	    c.r,
	    c.g,
	    c.b
	);

	return;
}


/*
 *	Saves configuration to filename.  Returns 0 on success
 *	and -1 on error.
 */
#define WRITE	bytes_written += fwrite(buf,\
                                        sizeof(char),\
                                        strlen(buf),\
                                        fp\
                );

int RCSaveToFile(char *filename)
{
	int i;
	char buf[CFG_STRING_MAX];
	FILE *fp;

        char tmp_path[PATH_MAX + NAME_MAX];
        char cwd[PATH_MAX];

        univ_entry_struct *univ_entry_ptr;

	int bytes_written = 0;



	/* Get current working dir. */
	getcwd(cwd, PATH_MAX);
	cwd[PATH_MAX - 1] = '\0';


        /* Open filename for writing. */
        fp = fopen(filename, "w");
        if(fp == NULL)
        {
            fprintf(stderr,
		"%s: Cannot open for writing.\n",
		filename
	    );
            return(-1);
        }


	/* ********************************************************** */
	/* Header. */
	sprintf(buf,
            "# %s Configuration File\n",
	    XSW_NAME
	);
	WRITE

        sprintf(buf,
            "# Version: %i.%i\n",
	    XSW_VERSION_MAJOR, XSW_VERSION_MINOR
        );
	WRITE

        sprintf(buf,   
	    "# You may manually edit this file.\n#\n\n\n"
        );
        WRITE


	/* Versions. */
        sprintf(buf,
            "# This configuration file version:\n\n"
        );
	WRITE

        sprintf(buf,
            "VersionMajor = %i\n",
	    XSW_VERSION_MAJOR
        );
	WRITE

        sprintf(buf,
            "VersionMinor = %i\n",
	    XSW_VERSION_MINOR
        );
	WRITE

        sprintf(buf, "\n\n");
	WRITE


        /* ********************************************************** */
	/* Directories and files. */
        sprintf(buf,
            "# Directories and files:\n\n"
        );
	WRITE

	/* ToplevelDir */
	if(ISPATHABSOLUTE(dname.toplevel))
	{
            sprintf(buf, "ToplevelDir = %s\n", dname.toplevel);
	}
	else
	{
	    /* ToplevelDir must be absolute, use default if not. */
	    sprintf(buf, "ToplevelDir = %s\n", DEF_XSW_TOPLEVEL_DIR);
	}
	WRITE

	/* ImagesDir */
	if(COMPARE_PARENT_PATHS(dname.images, dname.toplevel))
	{
	    /* Use relative path. */
            strncpy(tmp_path, dname.images, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
	    StripAbsolutePath(tmp_path);
            sprintf(buf, "ImagesDir = %s\n", tmp_path);
	}
	else
	{
            sprintf(buf, "ImagesDir = %s\n", dname.images);
	}
	WRITE

	/* SoundsDir */
        if(COMPARE_PARENT_PATHS(dname.sounds, dname.toplevel))
        {
            /* Use relative path. */
	    strncpy(tmp_path, dname.sounds, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
	    sprintf(buf, "SoundsDir = %s\n", tmp_path);
	}
        else
        {
	    sprintf(buf, "SoundsDir = %s\n", dname.sounds);
	}
	WRITE

	/* EtcDir */
        if(COMPARE_PARENT_PATHS(dname.etc, dname.toplevel))
	{
            /* Use relative path. */
            strncpy(tmp_path, dname.etc, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
	    sprintf(buf, "EtcDir = %s\n", tmp_path);
	}
	else
        {
            sprintf(buf, "EtcDir = %s\n", dname.etc);
        }
	WRITE

	/* SoundServerStartCommand */
        sprintf(buf,
            "SoundServerStartCommand = %s\n",
            sound.start_cmd
        );
        WRITE

#ifdef JS_SUPPORT
        /* Joystick caliberation file. */
        sprintf(buf,
            "JSCaliberationFile = %s\n",
            fname.js_calib
        );
	WRITE
#endif /* JS_SUPPORT */

	/* MainMenuConfFile */
        if(COMPARE_PARENT_PATHS(fname.main_menu_conf, dname.etc))
        {
            /* Use relative path. */
            strncpy(tmp_path, fname.main_menu_conf, PATH_MAX + NAME_MAX);
            tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
            sprintf(buf, "MainMenuConfFile = %s\n", tmp_path);
        }   
        else
        {
            sprintf(buf, "MainMenuConfFile = %s\n", fname.main_menu_conf);
        }
	WRITE

	/* OCSNamesFile */
        if(COMPARE_PARENT_PATHS(fname.ocsn, dname.etc))
        {
            /* Use relative path. */
            strncpy(tmp_path, fname.ocsn, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
            sprintf(buf, "OCSNamesFile = %s\n", tmp_path);
        }
        else
        {
	    sprintf(buf, "OCSNamesFile = %s\n", fname.ocsn);
	}
	WRITE

        /* ISRFile */
        if(COMPARE_PARENT_PATHS(fname.isr, dname.images))
        {
            /* Use relative path. */
            strncpy(tmp_path, fname.isr, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
            sprintf(buf, "ISRFile = %s\n", tmp_path);
        }
	else
        {
            sprintf(buf, "ISRFile = %s\n", fname.isr);
	}
	WRITE

        /* SoundSchemeFile */
        if(COMPARE_PARENT_PATHS(fname.sound_scheme, dname.sounds))
        {
            /* Use relative path. */
            strncpy(tmp_path, fname.sound_scheme, PATH_MAX + NAME_MAX);
	    tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
            sprintf(buf, "SoundSchemeFile = %s\n", tmp_path);
        }
	else
	{
            sprintf(buf, "SoundSchemeFile = %s\n", fname.sound_scheme);
	}
	WRITE

        /* LogFile */
        if(COMPARE_PARENT_PATHS(fname.log, cwd))
        {
            /* Use relative path. */
            strncpy(tmp_path, fname.log, PATH_MAX + NAME_MAX);
            tmp_path[PATH_MAX + NAME_MAX - 1] = '\0';
            StripAbsolutePath(tmp_path);
            sprintf(buf, "LogFile = %s\n", tmp_path);
        }
        else
        {   
            sprintf(buf, "LogFile = %s\n", fname.log);
        }
	WRITE

        sprintf(buf, "\n\n");
	WRITE


        /* ********************************************************* */
	/* Network parameters. */
        sprintf(buf,
            "# Network\n\n"
        );
	WRITE

        /* NetDeviceMaxLoad */
        sprintf(buf,
            "NetDeviceMaxLoad = %i\n",
            (int)loadstat.net_load_max
        );
	WRITE

        /* NetIntervalAutoTune */
        sprintf(buf,
            "NetIntervalAutoTune = %s\n",
            (auto_interval_tune.state == 1) ? "yes" : "no"
        );   
        WRITE

        /* NetIntervalTuneInterval */
        sprintf(buf,
	    "NetIntervalTuneInterval = %i\n",
            (int)auto_interval_tune.interval
        );
	WRITE

	/* DefaultServerAddress */
        sprintf(buf,
	    "DefaultServerAddress = %s\n",
            net_parms.address
        );
	WRITE

        /* DefaultServerPort */
        sprintf(buf,
	    "DefaultServerPort = %i\n",
            (int)net_parms.port
        );
	WRITE

        /* DefaultLoginName */
        sprintf(buf,
	    "DefaultLoginName = %s\n",
	    net_parms.login_name
        );
	WRITE

        /* DefaultLoginPassword */
        sprintf(buf,
	    "DefaultLoginPassword = %s\n",
            net_parms.login_password
        );
	WRITE

        sprintf(buf, "\n\n");
	WRITE


	/* ********************************************************* */
        /* General. */
        sprintf(buf,
	    "# General\n\n"
        );
        WRITE

        /* AutoMapEconomyWindow */
        sprintf(buf,
            "AutoMapEconomyWindow = %s\n",
            (option.auto_map_eco_win) ? "yes" : "no"
        );
	WRITE

	/* AutoMapUniverseListWindow */
        sprintf(buf,
            "AutoMapUniverseListWindow = %s\n",
            (option.auto_map_univ_list_win) ? "yes" : "no"
        );
        WRITE


        /* AutoViewscreenZoom */
        sprintf(buf,
            "AutoViewscreenZoom = %s\n",
            (option.auto_zoom == 1) ? "on" : "off"
        );
	WRITE

        /* ThrottleScopeMode */
	switch(option.throttle_mode)
	{
	  case THROTTLE_MODE_BIDIRECTIONAL:
	    sprintf(buf, "ThrottleScopeMode = bidirectional\n");
	    break;

	  default:
	    sprintf(buf, "ThrottleScopeMode = normal\n");
            break;
	}
	WRITE

        /* LocalUpdates */
        sprintf(buf,
            "LocalUpdates = %s\n",
            (option.local_updates) ? "on" : "off"
        );
	WRITE

        /* ViewScreenMarks (was DisplayHeadingArrow) */
        sprintf(buf,
            "ViewScreenMarks = %s\n",
	    (option.show_viewscreen_marks) ? "yes" : "no"
        );
	WRITE

        /* ViewScreenLabels */
        sprintf(buf,
            "ViewScreenLabels = %i\n",
	    option.show_viewscreen_labels
        );
	WRITE

	/* ShowLensFlares */
        sprintf(buf,
            "ShowLensFlares = %s\n",
            ((option.show_lens_flares) ? "yes" : "no")
        );
        WRITE

        /* ShowStrobeGlow */
        sprintf(buf,
            "ShowStrobeGlow = %s\n",
            ((option.show_strobe_glow) ? "yes" : "no")
        );
        WRITE

        /* ShowNebulaGlow */
        sprintf(buf,
            "ShowNebulaGlow = %s\n",
            ((option.show_nebula_glow) ? "yes" : "no")
        );
        WRITE

        /* FormalNameLabels */
        sprintf(buf,
            "FormalNameLabels = %i\n",
            option.show_formal_label
        );
	WRITE

	/* NotifyScannerContacts */
	sprintf(buf,
            "NotifyScannerContacts = %s\n",
            (option.notify_scanner_contacts) ? "on" : "off"
        );
        WRITE

	/* ShowNetErrors */
	sprintf(buf,
            "ShowNetErrors = %s\n",
            (option.show_net_errors) ? "yes" : "no"
        );
	WRITE

        /* ShowServerErrors */
        sprintf(buf,
            "ShowServerErrors = %s\n",
            (option.show_server_errors) ? "yes" : "no"
        );
        WRITE
 
	/* Sounds */
        sprintf(buf,  
            "Sounds = %i\n",
            option.sounds
        );
	WRITE

        /* SoundServerArgument */
        sprintf(buf,
            "SoundServerConnectionArgument = %s\n",
            sound.con_arg
        );
        WRITE

        /* ControlType */
	switch(option.controller)
	{
#ifdef JS_SUPPORT  
	  case CONTROLLER_JOYSTICK:
	    sprintf(buf, "ControlType = joystick\n");
	    break;
#endif /* JS_SUPPORT */

	  case CONTROLLER_POINTER:
	    sprintf(buf, "ControlType = pointer\n");
            break;

	  default:
            sprintf(buf, "ControlType = keyboard\n");
	    break;
	}
	WRITE

        /* Units */
	switch(option.units)
	{
	  case XSW_UNITS_XSW:
	    sprintf(buf,
                "Units = XSW\n"
            );
	    break;

          case XSW_UNITS_METRIC:
            sprintf(buf,
                "Units = AstroMetric\n"
            );
            break;

          case XSW_UNITS_ENGLISH:
            sprintf(buf,
                "Units = English\n"
            );
            break;

          default:
            sprintf(buf,
                "Units = XSW\n"
            );
            break;
	}
	WRITE

        /* AsyncImageLoading */
        sprintf(buf,
            "AsyncImageLoading = %s\n",
            (option.async_image_loading == 1) ? "yes" : "no"
        );
        WRITE

        /* AsyncImageLoadPixels */
        sprintf(buf,
            "AsyncImageLoadPixels = %i\n",
            option.async_image_pixels
        );
        WRITE


        sprintf(buf, "\n\n");
	WRITE


        /* ********************************************************* */
	/* Universes list. */

        sprintf(buf,
	    "# Bookmarked universes:\n\n"
        );
	WRITE

	/*   Save the universe entries by the order that is on the
         *   universe list.
	 */
	for(i = 0; i < univ_list_win.list.total_entries; i++)
	{
	    /* Skip NULL list entries. */
	    if(univ_list_win.list.entry[i] == NULL)
		continue;
	    if(univ_list_win.list.entry[i]->data_ptr == NULL)
                continue;

	    /* Get universe entry pointer from data pointer on list. */
	    univ_entry_ptr = (univ_entry_struct *)
		univ_list_win.list.entry[i]->data_ptr;


	    sprintf(buf, "BeginUniverseEntry\n");
            WRITE

	    /* Alias */
	    if(univ_entry_ptr->alias != NULL)
	    {
                sprintf(buf,
		    "    Alias = %s\n",
		    univ_list_win.list.entry[i]->name	/* Use name on list item. */
		);
                WRITE
	    }
	    /* URL */
	    if(univ_entry_ptr->url != NULL)
	    {
                sprintf(buf,
		    "    URL = %s\n",
		    univ_entry_ptr->url
		);
		WRITE
	    }
	    /* LastConnected */
            sprintf(buf,
		"    LastConnected = %i\n",
                (int)univ_entry_ptr->last_connected
	    );
            WRITE
	    /* Comments */
	    if(univ_entry_ptr->comments != NULL)
	    {
                sprintf(buf, "    Comments = ");
	        strncat(
		    buf,
		    univ_entry_ptr->comments,
		    MAX(CFG_STRING_MAX - 20, 0)
		);
		WRITE

                sprintf(buf, "\n");
		WRITE
	    }

            sprintf(buf, "EndUniverseEntry\n\n");
            WRITE
	}

        sprintf(buf, "\n");
	WRITE


        /* ********************************************************* */ 
        /* Quick Menu Entries. */
        sprintf(buf,
	    "# Quick menu entries:\n\n"
        );
	WRITE
                
        for(i = 0; i < total_qmenu_entries; i++)
        {
 	    if(qmenu_entry[i] == NULL)
		continue;

	    sprintf(buf, "BeginQuickMenuItem\n");
	    WRITE

            sprintf(buf, "    Name = %s\n",
		qmenu_entry[i]->name
	    );
	    WRITE

            sprintf(buf, "    Type = %i\n",  
                qmenu_entry[i]->type
	    );
	    WRITE

            sprintf(buf, "    ActionCode = %i\n",
                qmenu_entry[i]->op_code
	    );
	    WRITE
 
            sprintf(buf, "EndQuickMenuItem\n\n");
	    WRITE
	}

        sprintf(buf, "\n");
	WRITE


        /* ********************************************************* */ 
        /* Key Mappings. */
        sprintf(buf,
	    "# Key Mappings\n\n"
	);
	WRITE

        sprintf(buf, "BeginKeyMap\n");
	WRITE

	for(i = 0; i < TOTAL_XSW_KEYMAPS; i++)
	{
	    /* Write keymap, use conical name. */
	    sprintf(
		buf,
		"    %s = %i\n",
		xsw_keymap_name[i],
		xsw_keymap[i].keycode
	    );
	    WRITE
	}

        sprintf(buf, "EndKeyMap\n\n\n");
	WRITE


        /* ********************************************************* */
        /* Colors. */

	/* Colors: General text. */
        sprintf(buf, "# Colors: General Text\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorStandardText", xsw_color.standard_text_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBoldText", xsw_color.bold_text_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorWithdrawnText", xsw_color.withdrawn_text_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE


        /* Colors: Bridge panel. */
        sprintf(buf, "# Colors: Bridge Panel\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorBPStandardText", xsw_color.bp_standard_text_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPBoldText", xsw_color.bp_bold_text_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPWithdrawnText", xsw_color.bp_withdrawn_text_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorBPLightOutline", xsw_color.bp_light_outline_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPNormalOutline", xsw_color.bp_normal_outline_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPDarkOutline", xsw_color.bp_dark_outline_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorBPWarning", xsw_color.bp_warning_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPDanger", xsw_color.bp_danger_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPCritical", xsw_color.bp_critical_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE


        /* Colors: Bridge panel outline. */
        sprintf(buf, "# Colors: Bridge Panel Outline\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorBPHull", xsw_color.bpol_hull_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPPower", xsw_color.bpol_power_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPVis", xsw_color.bpol_vis_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPShields", xsw_color.bpol_shields_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPDmgCtl", xsw_color.bpol_dmgctl_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPThrottle", xsw_color.bpol_throttle_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorBPThrottleRev", xsw_color.bpol_throttle_rev_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE  


	/* Colors: Viewscreen labels. */
        sprintf(buf, "# Colors: Viewscreen Labels\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorVSLabelBg", xsw_color.vs_label_bg_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorVSLabelFg", xsw_color.vs_label_fg_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE


        /* Colors: Scanner marks. */
        sprintf(buf, "# Colors: Scanner Markings\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorScMarkUnknown", xsw_color.scmark_unknown_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorScMarkLocked", xsw_color.scmark_locked_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorScMarkWeapon", xsw_color.scmark_weapon_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorScMarkHome", xsw_color.scmark_home_cv
        ); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorScMarkArea", xsw_color.scmark_area_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE


        /* Colors: Keymap query dialog. */
        sprintf(buf, "# Colors: Keymap Query Dialog\n"); WRITE

        RC_PRINT_COLOR_STRING(
            buf, "ColorKeymapQueryBg", xsw_color.keymap_query_bg_cv
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorKeymapQueryFg", xsw_color.keymap_query_fg_cv
        ); WRITE

        sprintf(buf, "\n"); WRITE


	/* Colors: Viewscreen graphics. */
        sprintf(buf, "# Colors: Viewscreen Graphics\n"); WRITE
	RC_PRINT_COLOR_STRING(
	    buf, "ColorHeadingArrow", xsw_color.heading_arrow
	); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorLockArrow", xsw_color.lock_arrow
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorLockCursor", xsw_color.lock_cursor
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorStreamGreen", xsw_color.stream_green
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorStreamOrange", xsw_color.stream_orange
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorStreamPurple", xsw_color.stream_purple
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorStreamYellow", xsw_color.stream_yellow
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorShieldBlue", xsw_color.shield_blue
        ); WRITE
        RC_PRINT_COLOR_STRING(
            buf, "ColorStarGlow", xsw_color.star_glow
        ); WRITE

        sprintf(buf, "\n\n");
	WRITE



	/* ********************************************************* */

	/* Close file. */
	fclose(fp);
	fp = NULL;


	return(0);
}
