/*
            S H I P W A R S   S E R V E R   M O N I T O R

   Copyright (C) 1997, 1998, 1999 WolfPack Development Organization.

 */

#ifndef MONITOR_H
#define MONITOR_H


#include <stdio.h>
#include <db.h>
#include <sys/types.h>
#include <limits.h>

#include "../include/os.h"

#include "../include/objects.h"

#include "../include/osw-x.h"
#include "../include/widget.h"

#include "mesgwin.h"


/*
 *	Program name and version:
 */
#define PROG_NAME	"Monitor"
#define PROG_VERSION	"1.14"

#define PROG_VERSION_MAJOR	1
#define PROG_VERSION_MINOR	14


/*
 *	Usage information:
 */
#define PROG_USAGE_MESSAGE "\
*** Note: This program is intended to be called by another program or\n\
*** script.\n\
\n\
Usage: monitor <address> <port> [options] [GUI_options]\n\
\n\
    <address> specifies the address to the swserv you want to\n\
    monitor.\n\
\n\
    <port> specifies the AUX Stats port number swserv is expecting\n\
    monitoring client connections from.\n\
\n\
    [options] can be any of the following:\n\
\n\
        -q                     Surpress stdout and stderr messages.\n\
	--images <path>        Specifies the images directory.\n\
        --server <path>        Specifies the server directory.\n\
        --help                 Prints (this) help screen and exits.\n\
        --version              Prints version information and exits.\n\
\n\
    [GUI_options] can be any options standard to your GUI, consult\n\
    your GUI's manual for available options.\n\
\n"


/*
 *      Copyright information:
 */
#define PROG_COPYRIGHT_MESG "\
Copyright (C) 1997, 1998, 1999 WolfPack Development Team.\n"


/*
 *	Draw amount codes:
 */
#define MON_DRAW_AMOUNT_COMPLETE	0
#define MON_DRAW_AMOUNT_READOUT		1
#define MON_DRAW_AMOUNT_BUTTONS		2



/*
 *	Runlevel:
 */
int runlevel;

/*
 *	Current time since midnight (in milliseconds):
 */
time_t cur_millitime;

/*
 *	Current systime (in seconds):
 */
time_t cur_systime;


/*
 *	Options:
 */
struct {

	char quiet_mode;	/* Do not print errors. */

} option;


/*
 *	Directories:
 */
struct {

	char images[PATH_MAX];
	char server[PATH_MAX];

} dname;

/*
 *	Timmers:
 *
 *	In milliseconds.
 */
struct {

	time_t	pipe_recv,
		systems_check;

} next;


/*
 *	Intervals:
 *
 *	In milliseconds.
 */
struct {

	time_t	pipe_recv,
		systems_check;

} interval;



/*
 *	Monitor colors:
 */
struct {

	pixel_t	readout_text,
		button_text,
		messages_text;

} mon_color;

/*
 *	Monitor fonts:
 */
struct {

	font_t	*readout,
		*button,
		*messages;

} mon_font;

/*
 *	Monitor images:
 */
struct {

	image_t *readout_bkg,
		*mesg_bkg;

	image_t *btn_unarmed,
		*btn_armed,
		*btn_highlight;

	image_t *warning;

	pixmap_t monitor_icon_pm;

} mon_image;

/*
 *	Cursors:
 */
struct {

	WCursor	*standard,
		*text;

} mon_cursor;

/*
 *	Monitor push button widget:
 */
#define MON_BTN_STATE_UNARMED		0
#define MON_BTN_STATE_ARMED		1
#define MON_BTN_STATE_HIGHLIGHTED	2

typedef struct {

	int x, y;
	unsigned int width, height;
	win_t parent;

	int state;
	char *label;

	void *client_data;
	int (*func_cb)(void *);

} mon_push_button_struct;


/*
 *	Monitor stats:
 */
typedef struct {

	char title[UNV_TITLE_MAX];

	time_t uptime;	/* How long it has been up in systime sec. */

	int	total_connections,
		guest_connections;

	int	total_objects;

	off_t	mem_total,
		mem_con,
                mem_obj;

	time_t	next_save,
		next_export;

	pid_t	pid;		/* PID of server. */

} mon_stats_struct;


/*
 *	Monitor structure:
 */
typedef struct {

	char map_state;
	char is_in_focus;
	int x, y;
	unsigned int width, height;
	int visibility_state;

        char address[HOST_NAME_MAX];
	int port;

	int socket;

	win_t toplevel;
	pixmap_t toplevel_buf;

	/* Buttons. */
	mon_push_button_struct	shutdown_btn,
				messages_btn;

	/* Monitor stats. */
	mon_stats_struct	stats;

	/* Associated message window. */
	mesgwin_struct		mesgwin;

} monitor_struct;

monitor_struct **monitor;
int total_monitors;

monitor_struct *delete_monitor;


/* ***************************************************************** */


/* In monalloc.c */
extern int MonIsAllocated(int n);
extern int MonAllocate();
extern void MonDelete(int n);
extern void MonDeleteAll();
extern void MonReclaim();

/* In moncb.c */
extern int MonShutdownPBCB(void *ptr);
extern int MonMessagesPBCB(void *ptr);

/* In monmanage.c */
extern int MonInit(monitor_struct *m, int argc, char *argv[]);
extern void MonDraw(monitor_struct *m, int amount);
extern int MonManage(monitor_struct *m, event_t *event);
extern int MonManageAll(event_t *event);
extern void MonMap(monitor_struct *m);
extern void MonUnmap(monitor_struct *m);
extern void MonDestroy(monitor_struct *m);

/* In monmacros.c */
extern int MonDoFetch(monitor_struct *m);


/* In main.c */
extern void MonitorSignalHandler(int s);
extern int MonitorScrollBarCB(scroll_bar_struct *sb);

extern int MonitorConnect(char *address, int port);

extern int MonitorCreateMonitor(int argc, char *argv[]);
extern void MonitorResetTimmers();

extern int MonitorInit(int argc, char *argv[]);
extern void MonitorManage();
extern void MonitorShutdown();

/* In timming.c */
extern long MilliTime(void);


#endif /* MONITOR_H */
