#include "swserv.h"
 
  
/*
 *      Front end to parse server command.
 */
int CmdHandleInput(int condescriptor, char *cmd)
{
        char *strptr;   
        long object_num;
        
        char sndbuf[CS_DATA_MAX_LEN];
        
        char stringa[CS_DATA_MAX_LEN];
        char stringb[CS_DATA_MAX_LEN];
        char arg[CS_DATA_MAX_LEN];
        char command[CS_DATA_MAX_LEN];
        
        int status;


        /* Command must be valid and contain data. */
	if((cmd == NULL) ? 1 : (cmd[0] == '\0'))
	    return(-1);

        /* Must be connected. */
        if(!ConIsConnected(condescriptor))
	    return(-1);

        /* Must be logged in (have valid object). */
        object_num = connection[condescriptor]->object_num;
        if(DBIsObjectGarbage(object_num))
        {
            NetSendLiveMessage(condescriptor,
                "You must be logged in to execute a server command."
            );

            return(-1); 
        }

        /* Copy command into stringa. */
        strncpy(stringa, cmd, CS_DATA_MAX_LEN);
        stringa[CS_DATA_MAX_LEN - 1] = '\0';
        
        /* Sanitize stringa. */   
        strncpy(stringb, stringa, CS_DATA_MAX_LEN);
        stringb[CS_DATA_MAX_LEN - 1] = '\0';
        StringStripSpaces(stringb);
        
        strncpy(stringa, stringb, CS_DATA_MAX_LEN);
        stringa[CS_DATA_MAX_LEN - 1] = '\0';

	strptr = strchr(stringa, '\n');
        if(strptr != NULL) *strptr = '\0';
	strptr = strchr(stringa, '\r');
        if(strptr != NULL) *strptr = '\0';


        /* ******************************************************** */
        /* Begin parsing. */
	strptr = strchr(stringa, ' ');
        if(strptr != NULL)
        {
            strncpy(arg, (strptr + 1), CS_DATA_MAX_LEN);   
            arg[CS_DATA_MAX_LEN - 1] = '\0';
            *strptr = '\0';
            strncpy(command, stringa, CS_DATA_MAX_LEN);
            command[CS_DATA_MAX_LEN - 1] = '\0';
        }
        else
        {
            strncpy(arg, "", CS_DATA_MAX_LEN);
            strncpy(command, stringa, CS_DATA_MAX_LEN);
            command[CS_DATA_MAX_LEN - 1] = '\0';
        }


        /* Log this. */
        sprintf(stringa,
            "%s: Executed: Command: `%s'  Argument: `%s'",
            DBGetFormalNameStr(object_num),
            command,
            arg
        );
        if(sysparm.log_general == 1)
            LogAppendLineFormatted(fname.primary_log, stringa);


        /* ****************************************************** */
        /* Handle command. */

        /* Debug. */
        if(!strcmp(command, "debug"))
        {
	    status = CmdDebug(condescriptor, arg);
        }
	/* Help. */
        else if(!strcmp(command, "help") ||
                !strcmp(command, "hel") ||
                !strcmp(command, "he") ||
                !strcmp(command, "h") ||
                !strcmp(command, "?")
        )
        {
            status = CmdHelp(condescriptor, arg);
        }
        /* Version. */ 
        else if(!strcmp(command, "version") ||
                !strcmp(command, "ver")
        )
        {
            status = CmdVersion(condescriptor, arg);
        } 
        /* Sysparm. */
        else if(!strcmp(command, "sysparm") ||
                !strcmp(command, "syspar") ||
                !strcmp(command, "syspa") ||
                !strcmp(command, "sysp") ||
                !strcmp(command, "tune")
        )
        {
            status = CmdSysparm(condescriptor, arg);
        }
        /* Memory. */
        else if(!strcmp(command, "memory") ||
                !strcmp(command, "memor") ||
                !strcmp(command, "memo") ||
                !strcmp(command, "mem")
        )
        {
            status = CmdMemory(condescriptor, arg);
        }
        /* Disk. */
        else if(!strcmp(command, "disk") ||
                !strcmp(command, "dis") ||
                !strcmp(command, "di")
        )
        {
            status = CmdDisk(condescriptor, arg);
        }
        /* Processes. */
        else if(!strcmp(command, "ps")) 
        {
            status = CmdPS(condescriptor, arg);
        }
        /* Kill process. */
        else if(!strcmp(command, "kill"))
        {
            status = CmdKill(condescriptor, arg);
        }
        /* Set. */
        else if(!strcmp(command, "set") ||
                !strcmp(command, "se") ||
                !strcmp(command, "s")
        )
        {
            status = CmdSet(condescriptor, arg);
        }
	/* Wall. */
        else if(!strcmp(command, "wall") ||
                !strcmp(command, "wal")
        )
        {
            status = CmdWall(condescriptor, arg);
        }
        /* Who. */
        else if(!strcmp(command, "WHO") ||
                !strcmp(command, "who")
        )
        {
            status = CmdWho(condescriptor, arg);
        }
        /* Netstat. */
        else if(!strcmp(command, "netstat") ||
                !strcmp(command, "netsta") ||
                !strcmp(command, "netst") ||
                !strcmp(command, "nets")
        )
        {
            status = CmdNetstat(condescriptor, arg);
        }
        /* Score. */
        else if(!strcmp(command, "score"))
        {
            status = CmdScore(condescriptor, arg);
        }
        /* ETA. */
        else if(!strcmp(command, "eta"))
        {
            status = CmdETA(condescriptor, arg);
        }
        /* Save. */
        else if(!strcmp(command, "save")) 
        {
            status = CmdSaveDB(condescriptor, arg);
        }
        /* Shutdown. */
        else if(!strcmp(command, "shutdown"))
        {
            status = CmdShutdown(condescriptor, arg);
        }
        /* Boot. */
        else if(!strcmp(command, "boot"))
        {
            status = CmdBoot(condescriptor, arg);
        }
        /* Sync time. */
        else if(!strcmp(command, "synctime") ||
                !strcmp(command, "sync")
        )
        {
            status = CmdSyncTime(condescriptor, arg);
        }
        /* ID. */
        else if(!strcmp(command, "id"))
        {
            status = CmdID(condescriptor, arg);
        }
        /* Create object. */
        else if(!strcmp(command, "create"))
        {
            status = CmdCreate(condescriptor, arg);
        }
        /* Create player. */
        else if(!strcmp(command, "createplayer") ||
                !strcmp(command, "createplaye") ||
                !strcmp(command, "createplay") ||
                !strcmp(command, "createpla") ||
                !strcmp(command, "createpl") ||
                !strcmp(command, "createp")
        )
        {
            status = CmdCreatePlayer(condescriptor, arg);
        }
        /* Chown object. */
        else if(!strcmp(command, "chown") ||
                !strcmp(command, "chow") ||
                !strcmp(command, "cho") ||
                !strcmp(command, "ch")
        )
        {
            status = CmdChown(condescriptor, arg);
        }
        /* Recycle object. */
        else if( !strcmp(command, "recycle") ||
                 !strcmp(command, "recycl") ||
                 !strcmp(command, "recyc") ||
                 !strcmp(command, "recy") ||
                 !strcmp(command, "rec")
        )
        {
            status = CmdRecycle(condescriptor, arg);
        }
        /* Unrecycle object. */
        else if(!strcmp(command, "unrecycle") ||
                !strcmp(command, "unrecycl") ||
                !strcmp(command, "unrecyc") ||
                !strcmp(command, "unrecy") ||
                !strcmp(command, "unrec")
        )
        {
           status = CmdUnrecycle(condescriptor, arg);
        }
        /* Recycle player. */
        else if(!strcmp(command, "recycleplayer") ||
                !strcmp(command, "recyclep")
        )
        {
            status = CmdRecyclePlayer(condescriptor, arg);
        }
        /* Examine object. */
        else if(!strcmp(command, "examine") ||
                !strcmp(command, "examin") ||
                !strcmp(command, "exami") ||
                !strcmp(command, "exam") ||
                !strcmp(command, "exa") ||
                !strcmp(command, "ex")
        )
        {
            status = CmdExamine(condescriptor, arg);
        }
        /* Find. */
        else if(!strcmp(command, "find") ||
                !strcmp(command, "fin") ||
                !strcmp(command, "fi") ||
                !strcmp(command, "f")
        )
        {
            status = CmdFind(condescriptor, arg);
        }
        /* Eco product create. */
        else if(!strcmp(command, "ecoproductcreate") ||
                !strcmp(command, "ecoprodcreate") ||
                !strcmp(command, "ecoprodcreat") ||
                !strcmp(command, "ecoprodcrea") ||
                !strcmp(command, "ecoprodcre") ||
                !strcmp(command, "ecoprodcr") ||
                !strcmp(command, "ecoprodc")
        )
        {
            status = CmdEcoProductCreate(condescriptor, arg);
        }
        /* Eco product create. */
        else if(!strcmp(command, "ecoproductset") ||
                !strcmp(command, "ecoprodset") ||
                !strcmp(command, "ecoprodse") ||
                !strcmp(command, "ecoprods")
        )
        {
            status = CmdEcoProductSet(condescriptor, arg);
        }
	/* Eco product delete. */
        else if(!strcmp(command, "ecoproductdelete") ||
                !strcmp(command, "ecoproddelete") ||
                !strcmp(command, "ecoproddelet") ||
                !strcmp(command, "ecoproddele") ||
                !strcmp(command, "ecoproddel") ||
                !strcmp(command, "ecoprodde") ||
                !strcmp(command, "ecoprode")
        )
        {
            status = CmdEcoProductDelete(condescriptor, arg);
        }


        /* Test. */
        else if(!strcmp(command, "test")) 
        {
            status = CmdTest(condescriptor, arg);
        }

        /* Unknown command. */
        else 
        {
            status = -1;

            sprintf(sndbuf,
                "%s: no such command or macro.",
                command
            );
            NetSendLiveMessage(condescriptor, sndbuf);
        }


        return(status);
}
