/*
                            Widget: Push Button

	Functions:

	bool_t PBTNKEYMATCH(push_button_struct *btn, keycode_t keycode)

	void PBtnChangeLabel(
	        push_button_struct *btn,
	        unsigned int width, 
	        unsigned int height,
	        char *label,
	        char label_align,
	        image_t *image
	)

	int PBtnInit(
                push_button_struct *btn,
                win_t parent,
                int x, int y,
                unsigned int width,
                unsigned int height,
                char *label,
                char label_align,
                image_t *image,
		void *client_data,
                int (*func_cb)(void *)
	)
	int PBtnSetHintMessage(
		push_button_struct *btn,
		char *mesg
	)
	int PBtnSetHotKeys(
		push_button_struct *btn,
		char *hotkeys
	)

	int PBtnDraw(push_button_struct *btn)
	int PBtnManage(push_button_struct *btn, event_t *xevent)
	void PBtnMap(push_button_struct *btn)
	void PBtnUnmap(push_button_struct *btn)
	void PBtnDestroy(push_button_struct *btn)

 */

#include "../include/widget.h"


/* Default sizes. */
#define PBTN_DEFAULT_WIDTH	80
#define	PBTN_DEFAULT_HEIGHT	30

/* Character dimensions, in pixels. */
#define PBTN_CHAR_WIDTH		7
#define PBTN_CHAR_HEIGHT	14

/* All purpose margin value.  In pixels. */
#define PBTN_MARGIN		2



bool_t PBTNKEYMATCH(push_button_struct *btn, keycode_t keycode)
{
	char c;

	if(btn == NULL) return(False);

	c = OSWGetASCIIFromKeyCode(keycode, False, False, False);
	if(c == '\0') return(False);

	if(strchr(btn->hotkey, c) != NULL)
	    return(True);
	else
	    return(False);
}



void PBtnChangeLabel(
        push_button_struct *btn,
        unsigned int width,
        unsigned int height,
        char *label,
        char label_align,
        image_t *image
)
{
	image_t *tmp_img;


	if(!IDC() ||
           (btn == NULL)
	)
	    return;

	if(btn->toplevel == 0)
	    return;


	/* *********************************************************** */

	/* Set new alignment code. */
	btn->label_align = label_align;

	/* Calculate needed width and height. */
        if(width == 0)
        {
            if(image != NULL)
            {
                width = (int)image->width + (2 * PBTN_MARGIN);
            }
            if(label != NULL)
            {
                width = ((int)strlen(label) * PBTN_CHAR_WIDTH) +
                    (2 * PBTN_MARGIN);
            }
            if(width == 0)
            {
                width = PBTN_DEFAULT_WIDTH;
            }
        }
        if((int)width < (2 * PBTN_MARGIN))
            width = 2 * PBTN_MARGIN;
        if((int)width > (int)osw_gui[0].display_width)
            width = osw_gui[0].display_width;   

        if(height == 0)  
        {
            if(image != NULL)
            {
                height = (int)image->height + (2 * PBTN_MARGIN);
            }
            if(label != NULL)
            {
                height = PBTN_CHAR_HEIGHT + (2 * PBTN_MARGIN);
            }
            if(height == 0)
            {
                height = PBTN_DEFAULT_HEIGHT;
            }
        }
        if((int)height < (2 * PBTN_MARGIN))
            height = 2 * PBTN_MARGIN;
        if((int)height > (int)osw_gui[0].display_height)
            height = osw_gui[0].display_height;


	/* Resize window. */
	OSWResizeWindow(btn->toplevel, width, height);
	btn->width = width;
	btn->height = height;

	/* Destroy and recreate pixmap buffer. */
	OSWDestroyPixmap(&btn->toplevel_buf);
	if(
	    OSWCreatePixmap(
		&btn->toplevel_buf,
		width,
		height
	    )
	)
	    return;


	/* Recreate button images. */
	OSWDestroyImage(&btn->armed_img);
	OSWDestroyImage(&btn->unarmed_img);
        OSWDestroyImage(&btn->highlighted_img);

	if(widget_global.btn_unarmed_img != NULL)
	{
	    if(
		OSWCreateImage(&tmp_img, width, height)
	    )
		return;
	    WidgetResizeImageBuffer(
		osw_gui[0].depth,
		tmp_img->data,
		widget_global.btn_unarmed_img->data,
		tmp_img->width, tmp_img->height,
		widget_global.btn_unarmed_img->width,
		widget_global.btn_unarmed_img->height
	    );
	    btn->unarmed_img = tmp_img;
	}
        if(widget_global.btn_armed_img != NULL)
        {
            if(
                OSWCreateImage(&tmp_img, width, height) 
            ) 
                return;
            WidgetResizeImageBuffer(
                osw_gui[0].depth,
                tmp_img->data,
                widget_global.btn_armed_img->data,
                tmp_img->width, tmp_img->height,
                widget_global.btn_armed_img->width,
		widget_global.btn_armed_img->height
            );
            btn->armed_img = tmp_img;
        }
        if(widget_global.btn_highlighted_img != NULL)
        {
            if(
                OSWCreateImage(&tmp_img, width, height)
            )
                return;
            WidgetResizeImageBuffer(
                osw_gui[0].depth,
                tmp_img->data,
                widget_global.btn_highlighted_img->data,
                tmp_img->width, tmp_img->height,
                widget_global.btn_highlighted_img->width,
		widget_global.btn_highlighted_img->height
            );
            btn->highlighted_img = tmp_img;
        }


	/* Set new label image. */
	btn->image = image;

	/* Set label text. */
	if(label != NULL)
	{
	    free(btn->label); btn->label = NULL;
	    btn->label = (char *)calloc(1, (strlen(label) + 1) *
                sizeof(char));
	    if(btn->label == NULL)
		return;
	    strcpy(btn->label, label);
	}
	else
	{
	    free(btn->label); btn->label = NULL;
	}


	return;
}



int PBtnInit(
	push_button_struct *btn,
	win_t parent,
	int x, int y,
	unsigned int width,
	unsigned int height,
	char *label,
	char label_align,
        image_t *image,
	void *client_data,
	int (*func_cb)(void *)
)
{
	int len;
	pixmap_t pixmap;
	image_t *tmp_img;


	if(!IDC() ||
	   (parent == 0) ||
           (btn == NULL)
	)
	    return(-1);


	/* Sanitize width and height. */
	if(width == 0)
	{
	    if(image != NULL)
	    {
		width = (int)image->width + (2 * PBTN_MARGIN);
	    }
	    if(label != NULL)
	    {
		width = ((int)strlen(label) * PBTN_CHAR_WIDTH) +
                    (2 * PBTN_MARGIN);
	    }
	    if(width == 0)
	    {
		width = PBTN_DEFAULT_WIDTH;
	    }
	}
        if((int)width < (2 * PBTN_MARGIN))
            width = 2 * PBTN_MARGIN;
        if((int)width > (int)osw_gui[0].display_width)
            width = osw_gui[0].display_width;

        if(height == 0)
        {
            if(image != NULL)  
            {
                height = (int)image->height + (2 * PBTN_MARGIN);
            }
            if(label != NULL)
            {
                height = PBTN_CHAR_HEIGHT + (2 * PBTN_MARGIN);
            }
            if(height == 0)
            {
                height = PBTN_DEFAULT_HEIGHT;
            }   
        }
        if((int)height < (2 * PBTN_MARGIN))
            height = 2 * PBTN_MARGIN;
        if((int)height > (int)osw_gui[0].display_height)
            height = osw_gui[0].display_height;

	/* Width or height cannot be odd. */
	if(IS_NUM_ODD(width)) width += 1;
	if(IS_NUM_ODD(height)) height += 1;


	/* ********************************************************* */

	/* Reset values. */
	btn->map_state = 0;
	btn->x = x;
	btn->y = y;
	btn->width = width;
	btn->height = height;
	btn->visibility_state = VisibilityFullyObscured;
	btn->disabled = False;
	btn->font = OSWQueryCurrentFont();

	btn->state = PBTN_UNARMED;

	bzero(btn->hotkey, PBTN_MAX_HOTKEYS + 1);


	/* Record parent. */
	btn->parent = parent;


	/* Create toplevel window. */
	if(
	    OSWCreateWindow(
	        &btn->toplevel,
	        parent,
	        btn->x, btn->y,
	        btn->width, btn->height
	    )
	)
	    return(-1);

	OSWSetWindowInput(btn->toplevel, KeyPressMask |
            KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
            ExposureMask | VisibilityChangeMask | EnterWindowMask |
            LeaveWindowMask
	);

	if(widget_global.btn_unarmed_img != NULL)
	{
            if(OSWCreateImage(
                &tmp_img, btn->width, btn->height
            ))
                return(-1);
            WidgetResizeImageBuffer( 
                osw_gui[0].depth,
                tmp_img->data,
                widget_global.btn_unarmed_img->data,
                tmp_img->width, tmp_img->height,
                widget_global.btn_unarmed_img->width,
                widget_global.btn_unarmed_img->height 
            );
	    pixmap = OSWCreatePixmapFromImage(tmp_img);
	    OSWDestroyImage(&tmp_img);




	    OSWSetWindowBkg(btn->toplevel, 0, pixmap);
	    OSWDestroyPixmap(&pixmap);
	}
	else
	{
	    OSWSetWindowBkg(btn->toplevel, 0, widget_global.std_bkg_pm);
	}

	/* ************************************************************* */

	/* Set label alignment. */
	btn->label_align = label_align;


	/* Allocate label string. */
	if(label != NULL)
	{
	    len = strlen(label);
	    btn->label = (char *)calloc(1, (len + 1) * sizeof(char));
	    strcpy(btn->label, label);
	}
	else
	{
	    btn->label = NULL;
	}


	/* Set label image. */
	btn->image = image;


	/* Set callback function pointer (can be NULL). */
	btn->client_data = client_data;
	btn->func_cb = func_cb;


        /* Add button to wdiget regeristry. */
        WidgetRegAdd((void *)btn, WTYPE_CODE_PUSHBUTTON);


	return(0);
}


/*
 *	Sets hint message.
 */
int PBtnSetHintMessage(
	push_button_struct *btn,
        char *mesg
)
{
	if((btn == NULL) ||
           (mesg == NULL)
	)
	    return(-1);

	/* Toplevel of button structure must be initialized. */
	if(btn->toplevel == 0)
	    return(-3);

	/* Allocate and set hint window message. */
	if(
	    HintWinAddMessage(
	        btn->toplevel,
	        0,	/* Parent (no longer used). */
	        0, 0,	/* x, y (no longer used). */
	        mesg
	    ) < 0
	)
	    return(-1);
	else
	    return(0);
}



int PBtnSetHotKeys(
	push_button_struct *btn,
        char *hotkeys
)
{
	if(btn == NULL)
	    return(-1);

	if(hotkeys == NULL)
	    return(-1);


	bzero(btn->hotkey, PBTN_MAX_HOTKEYS + 1);
	strcpy(btn->hotkey, hotkeys);


	return(0);
}



int PBtnDraw(push_button_struct *btn)
{
	int mapped = 0;
	int i, n;
	image_t *tmp_img;
	int x, y, len;
	win_attr_t wattr;
	font_t *prev_font;


        if(!IDC() ||
           (btn == NULL)
        )
            return(-1);


	/* Do not draw if mapped and fully obscured. */
/*
	if((btn->map_state == 1) &&
           (btn->visibility_state == VisibilityFullyObscured)
	)
	    return(0);
*/


        /* ********************************************************* */

	/* Get toplevel attributes. */
	OSWGetWindowAttributes(btn->toplevel, &wattr);

	/* Map as needed. */
	if(btn->map_state == 0)
	{
            /* Create toplevel buffer. */
	    if(btn->toplevel_buf == 0)
	    {
                if(OSWCreatePixmap(
		    &btn->toplevel_buf,  
                    btn->width, btn->height
                ))
                    return(-1);
	    }

            /* Create button background images. */ 
            if((widget_global.btn_unarmed_img != NULL) &&
               (btn->unarmed_img == NULL)
	    )
            {
                if(OSWCreateImage(&tmp_img, btn->width, btn->height))
                    return(-1);
                WidgetResizeImageBuffer(
                    osw_gui[0].depth,
                    tmp_img->data,
                    widget_global.btn_unarmed_img->data,
                    tmp_img->width, tmp_img->height,   
                    widget_global.btn_unarmed_img->width,
                    widget_global.btn_unarmed_img->height
                );
                btn->unarmed_img = tmp_img;
            }
            if((widget_global.btn_armed_img != NULL) &&
               (btn->armed_img == NULL)
            )
            {  
                if(OSWCreateImage(&tmp_img, btn->width, btn->height))
                    return(-1);
                WidgetResizeImageBuffer(
                    osw_gui[0].depth,
                    tmp_img->data,
                    widget_global.btn_armed_img->data,
                    tmp_img->width, tmp_img->height,
                    widget_global.btn_armed_img->width,
                    widget_global.btn_armed_img->height
                );
                btn->armed_img = tmp_img;
            }
            if((widget_global.btn_highlighted_img != NULL) &&
               (btn->highlighted_img == NULL)
            )
            {
                if(OSWCreateImage(&tmp_img, btn->width, btn->height))
                    return(-1);
                WidgetResizeImageBuffer( 
                    osw_gui[0].depth,
                    tmp_img->data,
                    widget_global.btn_highlighted_img->data,
                    tmp_img->width, tmp_img->height,
                    widget_global.btn_highlighted_img->width,
                    widget_global.btn_highlighted_img->height
                );
                btn->highlighted_img = tmp_img;
            }


	    /* Map the button. */
	    OSWMapRaised(btn->toplevel);

	    btn->map_state = 1;
	    btn->visibility_state = VisibilityUnobscured;

	    mapped = 1;
	}


	/* ********************************************************* */

	prev_font = OSWQueryCurrentFont();
	OSWSetFont(btn->font);

	/* Draw button background. */
	if(widget_global.force_mono)
	{
	    OSWClearPixmap(btn->toplevel_buf, wattr.width, wattr.height,
		osw_gui[0].black_pix);
	    OSWSetFgPix(osw_gui[0].white_pix);
	    OSWDrawRectangle(btn->toplevel_buf, 0, 0,
		(int)wattr.width - 1, (int)wattr.height - 1);
	}
	else
	{
            if(btn->state == PBTN_ARMED)
            {
                OSWPutImageToDrawable(btn->armed_img,
                    (drawable_t)btn->toplevel_buf
                );
            }
	    else if(btn->state == PBTN_HIGHLIGHTED)
            {
                OSWPutImageToDrawable(btn->highlighted_img,
                    (drawable_t)btn->toplevel_buf
                );
            }
            else
            {
                OSWPutImageToDrawable(btn->unarmed_img,
                    (drawable_t)btn->toplevel_buf
                );
            }
	}


	/* Draw image label. */
	if((btn->image != NULL) &&
           !widget_global.force_mono
	)
	{
            /* Calculate x position. */
            switch(btn->label_align)
            {
              case PBTN_TALIGN_LEFT:
                x = PBTN_MARGIN;
                break;
         
              case PBTN_TALIGN_RIGHT:
                x = (int)wattr.width - (int)btn->image->width - PBTN_MARGIN;
                break;
 
              /* Default to centered. */
              default:
                x = ((int)wattr.width / 2) -
                    ((int)btn->image->width / 2);

                break;
	    }

            /* Calculate y position. */
            y = ((int)wattr.height / 2) - ((int)btn->image->height / 2);

	    WidgetPutImageNormal(
		(drawable_t)btn->toplevel_buf,
		btn->image,
		x, y,
		True
	    );
	}



	/* Draw text label. */
	len = (btn->label == NULL) ? 0 : strlen(btn->label);
	if(len > 0)
	{
	    /* Calculate x position. */
	    switch(btn->label_align)
	    {
	      case PBTN_TALIGN_LEFT:
		x = PBTN_MARGIN;
		break;

	      case PBTN_TALIGN_RIGHT:
		x = (int)wattr.width - (len * PBTN_CHAR_WIDTH) -
                    PBTN_MARGIN;
		break;

	      /* Default to centered. */
	      default:
		x = ((int)wattr.width / 2) -
		    (((int)len * PBTN_CHAR_WIDTH) / 2);
		break;
	    }

	    /* Calculate y position. */
	    y = ((int)wattr.height / 2) + 5;

	    if(widget_global.force_mono)
		OSWSetFgPix(osw_gui[0].white_pix);
	    else if(btn->disabled)
		OSWSetFgPix(widget_global.disabled_text_pix);
	    else
	        OSWSetFgPix(widget_global.normal_text_pix);
	    OSWDrawString(btn->toplevel_buf, x, y, btn->label);

	    /* Draw underline. */
	    for(n = 0; n < PBTN_MAX_HOTKEYS; n++)
	    {
                if(btn->hotkey[n] == '\0')
                    break;

		if(isblank(btn->hotkey[n]) ||
                   (btn->hotkey[n] == '\n') ||
                   (btn->hotkey[n] == '\r')
		)
		    continue;

	        for(i = 0; i < len; i++)
	        {
		    if(toupper(btn->label[i]) == toupper(btn->hotkey[n]))
		    {
			/* Draw underline for char i. */
			OSWDrawLine(btn->toplevel_buf,
			    x + (i * 8), y + 2,
			    x + (i * 8) + 6, y + 2
			);
			break;
		    }
		}
	    }
	}

	OSWPutBufferToWindow(btn->toplevel, btn->toplevel_buf);
	if(mapped)
	    OSWSetWindowBkg(btn->toplevel, 0, btn->toplevel_buf);

        OSWSetFont(prev_font);


	return(0);
}



int PBtnManage(push_button_struct *btn, event_t *xevent)
{
	static int xevents_handled;
	static win_attr_t wattr;   



        if((xevent == NULL) ||
           (btn == NULL)
        )
            return(0);


	/* Do not manage if not mapped. */
	if(btn->map_state == 0)
	    return(0);



	/* ************************************************************ */

        xevents_handled = 0;
	switch(xevent->type)
	{
          /* ********************************************************* */
	  case KeyPress:
	    if(PBTNKEYMATCH(btn, xevent->xkey.keycode))
	    {
		btn->state = PBTN_ARMED;
		OSWKBAutoRepeatOff();

		xevents_handled++;
	    }
	    break;

          /* ********************************************************* */
	  case KeyRelease:
            if(PBTNKEYMATCH(btn, xevent->xkey.keycode))
            {
                btn->state = PBTN_UNARMED;

		if(btn->func_cb != NULL)
		{
		    /* Use btn pointer as referance for now. */
                    btn->func_cb(btn->client_data);
		}

                OSWKBAutoRepeatOn();

                xevents_handled++;
            }
	    break;

          /* ********************************************************* */
          case Expose:
            if(xevent->xany.window == btn->toplevel)
	    {
		PBtnDraw(btn);

                xevents_handled++;
		return(xevents_handled);
	    }

            break;

	  /* ********************************************************* */
	  case ButtonPress:
            if(xevent->xany.window == btn->toplevel)
	    {
		btn->state = PBTN_ARMED;

                xevents_handled++;
	    }
	    break;

          /* ********************************************************* */ 
          case ButtonRelease:
            if(xevent->xany.window == btn->toplevel)
            {
		if(btn->state == PBTN_ARMED)
		{
                    OSWGetWindowAttributes(btn->toplevel, &wattr);

		    if(((int)xevent->xbutton.x >= 0) &&
                       ((int)xevent->xbutton.y >= 0) &&
                       ((int)xevent->xbutton.x < (int)wattr.width) &&
                       ((int)xevent->xbutton.y < (int)wattr.height) &&
                       (btn->func_cb != NULL)
		    )
		    {
			/* Use btn pointer as referance for now. */
		        btn->func_cb(btn->client_data);
		    }

		    btn->state = PBTN_HIGHLIGHTED;

		    /* Unschedual hints if it was schedualed. */
		    if(hint_win.next_map > 0)
		    {
			HintWinUnmap();
		    }

                    xevents_handled++;
		}
		else
		{
		    btn->state = PBTN_UNARMED;

		    /* Do not count as event. */
		}
            }
            break;

          /* ********************************************************* */
          case VisibilityNotify:
            if(xevent->xany.window == btn->toplevel)
            {
                btn->visibility_state = xevent->xvisibility.state;
                xevents_handled++;

                /* No need to continue, just return. */
                return(xevents_handled);
            }
            break;

	  /* ********************************************************* */
	  case EnterNotify:
            if(xevent->xany.window == btn->toplevel)
            {
		if(btn->state == PBTN_UNARMED)
		{
		    btn->state = PBTN_HIGHLIGHTED;

                    /* Schedual hints window map. */
                    HintWinSetSchedual(
                        widget_global.hintwin_map_delay,
                        btn->toplevel
                    );

                    xevents_handled++;
		}
            }
            break;

          /* ********************************************************* */
          case LeaveNotify:
            if(xevent->xany.window == btn->toplevel)
            {
		if((btn->state == PBTN_ARMED) ||
                   (btn->state == PBTN_HIGHLIGHTED)
		)
		{
		    btn->state = PBTN_UNARMED;

                    /* Unmap/unschedual hints window. */
                    HintWinUnmap();
         	}
		xevents_handled++;
            }
            break;
	}


	/* Draw as needed. */
	if(xevents_handled > 0)
	{
	    PBtnDraw(btn);
	}


	return(xevents_handled);
}



void PBtnMap(push_button_struct *btn)
{
        /* Error checks. */
        if(!IDC() ||
           (btn == NULL)
        )
            return;

        if(btn->toplevel == 0)
            return;


	/* Map and draw. */
	btn->map_state = 0;
	btn->state = PBTN_UNARMED;
	PBtnDraw(btn);


	return;
}



void PBtnUnmap(push_button_struct *btn)
{
        /* Error checks. */
        if(!IDC() ||
           (btn == NULL)
        )
            return;

        if(btn->toplevel == 0)
            return;


	/* Unmap. */
	btn->map_state = 0;
	btn->visibility_state = VisibilityFullyObscured;
	OSWUnmapWindow(btn->toplevel);


	/* Destroy toplevel buffer and images. */
	OSWDestroyPixmap(&btn->toplevel_buf);
	OSWDestroyImage(&btn->unarmed_img);
	OSWDestroyImage(&btn->armed_img);
        OSWDestroyImage(&btn->highlighted_img);


	return;
}



void PBtnDestroy(push_button_struct *btn)
{
	if(btn == NULL) return;


	/* Delete widget from regeristry. */
	WidgetRegDelete((void *)btn);


	if(IDC())
	{
	    /* Delete associated hint message. */
	    HintWinDeleteMessage(btn->toplevel);

	    /* Delete button windows and buffers. */
	    OSWDestroyPixmap(&btn->toplevel_buf);
	    OSWDestroyWindow(&btn->toplevel);

	    /* Destroy local button background images. */
	    OSWDestroyImage(&btn->unarmed_img);
	    OSWDestroyImage(&btn->armed_img);
            OSWDestroyImage(&btn->highlighted_img);
	}


        btn->map_state = 0;
        btn->x = 0;
        btn->y = 0;
        btn->width = 0;
        btn->height = 0;
        btn->visibility_state = VisibilityFullyObscured;
        btn->disabled = False;

        btn->parent = 0;
        btn->state = PBTN_UNARMED;
        
        free(btn->label);
        btn->label = NULL;

        btn->image = NULL;	/* Do not destroy image, client owned. */

        btn->label_align = PBTN_TALIGN_CENTER;

	btn->client_data = NULL;
        btn->func_cb = NULL;


	return;
}
