/*
                  Buffer Blitting: Standard Blitting Routines

           (Came from XShipWars client, may be out of date with it)

	Functions:

	BlitBufNormal8()
	BlitBufNormal16()
	BlitBufNormal32()
	BlitBufNormal()

	BlitBufAbsolute8()
	BlitBufAbsolute16()
	BlitBufAbsolute32()
	BlitBufAbsolute()

	BlitBufAdditive8()
	BlitBufAdditive16()
	BlitBufAdditive32()
	BlitBufAbsolute()

	BlitBufSolid8()
	BlitBufSolid16()
	BlitBufSolid32()
	BlitBufSolid()

	---

	These functions perform blitting of two buffers.
	A porition of the source buffer to a position on the destination
	buffer with `zooming' support.

	Zooming is currently limited to 0.01 to 1.00 ranges.

 */

#include "blitting.h"



/*
 *   Checks if tx, ty, sy, and sy are 'on the screen':
 *
 *      The variables beginning with a t means target, and respectively
 *	x, y, width, and height.
 *
 *      The variables beginning with a s means source, and respectively
 *      x, y, width, and height.
 */
#define ISPOSOFFSCREEN(tx,ty,tw,th,sx,sy,sw,sh) \
( (tx < 0) || (ty < 0) || (tx >= tw) || (ty >= th) ||\
(sx < 0) || (sy < 0) || (sx >= sw) || (sy >= sh) )


#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define MAX(a,b)        (((a) > (b)) ? (a) : (b))


/*
 *                          NORMAL BLITTING
 */
void BlitBufNormal8(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,   
        unsigned int src_copy_height,  
        double zoom,
        double vis
)
{
        double gamma; 
        int tar_x_col, tar_y_row;
        int src_x_col, src_y_row;

        unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL8 */
        int src_width_bytes;     /* src_width * BYTES_PER_PIXEL8 */

        int src_width_check;   /* src_x + src_copy_width */
        int src_height_check;  /* src_y + src_copy_height */

        u_int8_t *tar_buf_ptr;
        u_int8_t *src_buf_ptr;
        u_int8_t r, g, b;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) || 
           (src_width < 1) ||   
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;


        /* Sanitize zoom. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;


        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL8;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL8;

        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);

        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom); 

        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }

        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;

        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;


        /* ********************************************************* */
        /* Normal target to source. */
        if(vis >= VIS_FULL)
        {
            while(src_y_row < src_height_check)    
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;  
                        tar_y_row++;
                    }

                    continue;
                }


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int8_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL8)
                ]);

                /* Transparency check. */
                if(*src_buf_ptr == 0x00)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int8_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL8)
                ]);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = *src_buf_ptr;


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */   
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;   
                    tar_y_row++;
                }
            }
        }
        /* ********************************************************* */
        /* Upper threashold visibility. */
        else if(vis >= VIS_THRESH)  
        {
            gamma = (double)(vis - VIS_THRESH) /
                (double)(VIS_FULL - VIS_THRESH);

            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height) 
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;  
                    }

                    continue;
                }


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int8_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL8)
                ]);

                /* Transparency check. */
                if(*src_buf_ptr == 0x00)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }

                r = (u_int8_t)(((*src_buf_ptr & 0xE0) >> 5) * gamma);
                g = (u_int8_t)(((*src_buf_ptr & 0x1C) >> 2) * gamma);
                b = (u_int8_t)(((*src_buf_ptr & 0x03) << 2) * gamma);

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int8_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL8) 
                ]);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO8(r, g, b);


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }
            }
        }
        /* ********************************************************* */
        /* Lower threashold visibility. */
        else if(vis >= VIS_INVIS)
        {
            gamma = (double)(vis - VIS_INVIS) /
                (double)(VIS_THRESH - VIS_INVIS);
            gamma = 1 - gamma;  /* Flip. */

            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;   
                }


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int8_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL8)
                ]);

                /* Transparency check. */  
                if(*src_buf_ptr == 0x00)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int8_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL8)
                ]);

                r = (u_int8_t)(((*tar_buf_ptr & 0xE0) >> 5) * gamma);
                g = (u_int8_t)(((*tar_buf_ptr & 0x1C) >> 2) * gamma);
                b = (u_int8_t)(((*tar_buf_ptr & 0x03) << 2) * gamma);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO8(r, g, b);


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }
            }
        }


        return;
}


void BlitBufNormal16(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
	double zoom,
	double vis
)
{
	double gamma;
        int tar_x_col, tar_y_row;
        int src_x_col, src_y_row;

	unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL16 */
        int src_width_bytes;     /* src_width * BYTES_PER_PIXEL16 */

	int src_width_check;	/* src_x + src_copy_width */
	int src_height_check;	/* src_y + src_copy_height */

        u_int16_t *tar_buf_ptr;
	u_int16_t *src_buf_ptr;
	u_int8_t r, g, b;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;


	/* Sanitize zoom. */
	zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;


        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL16;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL16;

	/* Convert source units to 256 (for zooming). */
	src_x *= 256;
        src_y *= 256;
	src_copy_width = (int)src_copy_width * 256;
	src_copy_height = (int)src_copy_height * 256;
	src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);  
        src_height_check = (int)(src_y + (int)src_copy_height);

	/* Calculate source increments for zooming affect. */
	x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom);

	/* Sanitize target values. */
	if(tar_x < 0)
	{
	    src_x += (tar_x * -256 * (1 / zoom));
	    tar_x = 0;
	}
        if(tar_y < 0)
	{
	    src_y += (tar_y * -256 * (1 / zoom));
	    tar_y = 0;
	}

	if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
	if(src_height_check > (int)src_height)
	    src_height_check = src_height;

        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;
        

	/* ********************************************************* */
	/* Normal target to source. */
	if(vis >= VIS_FULL)
	{
            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
	        )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }


	        /* Get pointer in source buffer. */
	        src_buf_ptr = (u_int16_t *)(&src_buf[
		    ((src_y_row >> 8) * src_width_bytes) +
		    ((src_x_col >> 8) * BYTES_PER_PIXEL16)
	        ]);

	        /* Transparency check. */
                if(*src_buf_ptr == 0x0000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int16_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +  
                    (tar_x_col * BYTES_PER_PIXEL16)  
                ]);

	        /* Set set target buffer position value of source. */
	        *tar_buf_ptr = *src_buf_ptr;


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }
	    }
        }
        /* ********************************************************* */
        /* Upper threashold visibility. */
	else if(vis >= VIS_THRESH)
	{
	    gamma = (double)(vis - VIS_THRESH) /
		(double)(VIS_FULL - VIS_THRESH);

            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                    
                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                 
                        tar_x_col = tar_x;   
                        tar_y_row++;
                    }
                 
                    continue;
                }
                     
                        
                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int16_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL16)
                ]);

                /* Transparency check. */
                if(*src_buf_ptr == 0x0000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                    
                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }
                        
                    continue;
                }

		r = (u_int8_t)(((*src_buf_ptr & 0xF800) >> 8) * gamma);
                g = (u_int8_t)(((*src_buf_ptr & 0x07E0) >> 3) * gamma);
                b = (u_int8_t)(((*src_buf_ptr & 0x001F) << 3) * gamma);
                 
                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int16_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL16)
                ]);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO16(r, g, b);


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;
                    
                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {   
                    src_x_col = src_x;   
                    src_y_row += (int)y_pixels_inc; 
                     
                    tar_x_col = tar_x;
                    tar_y_row++;
                }    
            }
	}
        /* ********************************************************* */
	/* Lower threashold visibility. */
	else if(vis >= VIS_INVIS)
	{
            gamma = (double)(vis - VIS_INVIS) /
                (double)(VIS_THRESH - VIS_INVIS);
            gamma = 1 - gamma;	/* Flip. */

            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                 
                        tar_x_col = tar_x;   
                        tar_y_row++;
                    }

                    continue;
                }


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int16_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL16)
                ]);

                /* Transparency check. */
                if(*src_buf_ptr == 0x0000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }
                        
                    continue;
                }

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int16_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL16)
                ]);

                r = (u_int8_t)(((*tar_buf_ptr & 0xF800) >> 8) * gamma);
                g = (u_int8_t)(((*tar_buf_ptr & 0x07E0) >> 3) * gamma);
                b = (u_int8_t)(((*tar_buf_ptr & 0x001F) << 3) * gamma);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO16(r, g, b);


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {   
                    src_x_col = src_x;   
                    src_y_row += (int)y_pixels_inc; 

                    tar_x_col = tar_x;
                    tar_y_row++;
                }    
            }
	}


        return;
}



void BlitBufNormal32(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom,
	double vis
)
{
	register double gamma;
        register int tar_x_col, tar_y_row;
        register int src_x_col, src_y_row;
           
        register unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */
           
        register int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL32 */
        register int src_width_bytes;     /* src_width * BYTES_PER_PIXEL32 */
           
        register int src_width_check;   /* src_x + src_copy_width */
        register int src_height_check;  /* src_y + src_copy_height */

        register u_int32_t *tar_buf_ptr;
        register u_int32_t *src_buf_ptr;
	register u_int8_t a, r, g, b;
        
        
        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;
        
        
        /* Sanitize zoom. */
        zoom = (zoom > 1) ? 1 : zoom;
        
        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;

        
        /* ************************************************************* */
        
        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL32;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL32;
           
        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;
         
        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);
        
        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom); 
        
        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }
        
        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;
        
        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;
        
        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;
        
        
        /* ********************************************************* */
        /* Normal target to source. */
        if(vis >= VIS_FULL)
        {
            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;  
                        tar_y_row++;
                    }

                    continue;
                }


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int32_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL32)
                ]);

                /* Transparency check. */
                if(*src_buf_ptr == 0x00000000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;

                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;

                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int32_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL32)
                ]);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = *src_buf_ptr;


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;   
                    tar_y_row++;
                }
	    }
        }
        /* ********************************************************* */
        /* Upper threashold visibility. */
        else if(vis >= VIS_THRESH)
        {
            gamma = (double)(vis - VIS_THRESH) /
                (double)(VIS_FULL - VIS_THRESH);

            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height)
                )
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                     
                    /* Increment rows. */ 
                    if(src_x_col >= src_width_check)   
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }
                 
                    continue;
                }

        
                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int32_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL32)
                ]);
            
                /* Transparency check. */
                if(*src_buf_ptr == 0x00000000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                
                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }
                 
                    continue;
                }

                a = (u_int8_t)((*src_buf_ptr & 0xFF000000) >> 24);
                r = (u_int8_t)(((*src_buf_ptr & 0x00FF0000) >> 16) * gamma);
                g = (u_int8_t)(((*src_buf_ptr & 0x0000FF00) >> 8) * gamma);
                b = (u_int8_t)((*src_buf_ptr & 0x000000FF) * gamma);

                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int32_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL32)
                ]);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO32(a, r, g, b);
                 
                    
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;
                    
                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;
                     
                    tar_x_col = tar_x;
                    tar_y_row++;
                }
            }
        }
        /* ********************************************************* */
        /* Lower threashold visibility. */
        else if(vis >= VIS_INVIS)
        {
            gamma = (double)(vis - VIS_INVIS) /
                (double)(VIS_THRESH - VIS_INVIS);  
            gamma = 1 - gamma;  /* Flip. */
                    
            while(src_y_row < src_height_check)
            {
                /* Skip if target is off the screen. */
                if((tar_x_col >= (int)tar_width) ||
                   (tar_y_row >= (int)tar_height) 
                )
                {   
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                    
                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }
             
                    continue;
		}


                /* Get pointer in source buffer. */
                src_buf_ptr = (u_int32_t *)(&src_buf[
                    ((src_y_row >> 8) * src_width_bytes) +
                    ((src_x_col >> 8) * BYTES_PER_PIXEL32)
                ]); 

                /* Transparency check. */
                if(*src_buf_ptr == 0x00000000)
                {
                    /* Increment x colums. */
                    src_x_col += (int)x_pixels_inc;
                    tar_x_col++;
                    
                    /* Increment rows. */
                    if(src_x_col >= src_width_check)
                    {
                        src_x_col = src_x;
                        src_y_row += (int)y_pixels_inc;
                     
                        tar_x_col = tar_x;
                        tar_y_row++;
                    }

                    continue;
                }
             
                /* Get pointer in target buffer. */
                tar_buf_ptr = (u_int32_t *)(&tar_buf[
                    (tar_y_row * tar_width_bytes) +
                    (tar_x_col * BYTES_PER_PIXEL32)
                ]);

                a = (u_int8_t)((*tar_buf_ptr & 0xFF000000) >> 24);         
                r = (u_int8_t)(((*tar_buf_ptr & 0x00FF0000) >> 16) * gamma);
                g = (u_int8_t)(((*tar_buf_ptr & 0x0000FF00) >> 8) * gamma);
                b = (u_int8_t)((*tar_buf_ptr & 0x000000FF) * gamma);

                /* Set set target buffer position value of source. */
                *tar_buf_ptr = PACK8TO32(a, r, g, b);


                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }       
            }
        }


        return;
}



void BlitBufNormal(
        unsigned int d,
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom,			/* 0.1 to 1.0 supported. */
	double vis
)
{
	switch(d)
	{
	  case 32:
	    BlitBufNormal32(
                tar_buf, src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom, vis
	    );
	    break;

          case 24:
            BlitBufNormal32(
                tar_buf, src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width, 
                src_height,
                src_copy_width, 
                src_copy_height,
                zoom, vis
            );
            break;

          case 16:
            BlitBufNormal16(
                tar_buf, src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width, 
                src_height,
                src_copy_width, 
                src_copy_height,
                zoom, vis
            );
            break;

          case 8:
            BlitBufNormal8(
                tar_buf, src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom, vis
            );
            break;
	}

	return;
}



/*
 *                         ABSOLUTE BLITTING
 *
 *   Same as normal blitting except that it does not check for
 *   transparency.
 */
void BlitBufAbsolute8(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,   
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom                     /* 0.1 to 1.0 supported. */
)
{
        register int tar_x_col, tar_y_row;
        register int src_x_col, src_y_row;

        register unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        register int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL8 */
        register int src_width_bytes;     /* src_width * BYTES_PER_PIXEL8 */

        register int src_width_check;   /* src_x + src_copy_width */
        register int src_height_check;  /* src_y + src_copy_height */

        register u_int8_t *tar_buf_ptr;
        register u_int8_t *src_buf_ptr;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;


        /* Sanitize zoom. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;


        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL8;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL8;

        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);

        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom); 

        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }

        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;

        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;


        /* ********************************************************* */

        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;
 
                    tar_x_col = tar_x;
                    tar_y_row++;
                }

                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int8_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL8)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int8_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +
                (tar_x_col * BYTES_PER_PIXEL8)
            ]);
            
            /* Set set target buffer position value of source. */
            *tar_buf_ptr = *src_buf_ptr;


            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;

                tar_x_col = tar_x;
                tar_y_row++;
            }
        }


        return;
}


void BlitBufAbsolute16(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom                     /* 0.1 to 1.0 supported. */
)
{
        register int tar_x_col, tar_y_row;
        register int src_x_col, src_y_row;

        register unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        register int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL16 */
        register int src_width_bytes;     /* src_width * BYTES_PER_PIXEL16 */

        register int src_width_check;   /* src_x + src_copy_width */
        register int src_height_check;  /* src_y + src_copy_height */

        register u_int16_t *tar_buf_ptr;
        register u_int16_t *src_buf_ptr;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||   
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;
 
 
        /* Sanitize zoom. */
        zoom = (zoom > 1) ? 1 : zoom;
 
        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;
        
        
        /* ************************************************************* */
        
        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL16;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL16;
           
        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;
         
        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);
        
        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom); 
        
        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }
        
        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;
 
        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;
        
        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;
        
 
        /* ********************************************************* */
        
        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;
        
                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;
        
                    tar_x_col = tar_x;  
                    tar_y_row++;
                }
        
                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int16_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL16)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int16_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +
                (tar_x_col * BYTES_PER_PIXEL16)
            ]);

            /* Set set target buffer position value of source. */
            *tar_buf_ptr = *src_buf_ptr;


            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;
             
                tar_x_col = tar_x;   
                tar_y_row++;
            }
        }


        return;
}



void BlitBufAbsolute32(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom			/* 0.1 to 1.0 supported. */
)
{
        register int tar_x_col, tar_y_row;
        register int src_x_col, src_y_row;

        register unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        register int tar_width_bytes;     /* tar_width * BYTES_PER_PIXEL32 */
        register int src_width_bytes;     /* src_width * BYTES_PER_PIXEL32 */

        register int src_width_check;   /* src_x + src_copy_width */
        register int src_height_check;  /* src_y + src_copy_height */

        register u_int32_t *tar_buf_ptr;
        register u_int32_t *src_buf_ptr;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||   
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;


        /* Sanitize zoom. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;
        
        
        /* ************************************************************* */
        
        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL32;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL32;
           
        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;
         
        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);
        
        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom); 
        
        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }
        
        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;
 
        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;
        
        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;
        
 
        /* ********************************************************* */
        
        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;
        
                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;
        
                    tar_x_col = tar_x;  
                    tar_y_row++;
                }
        
                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int32_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL32)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int32_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +
                (tar_x_col * BYTES_PER_PIXEL32)
            ]);

            /* Set set target buffer position value of source. */
            *tar_buf_ptr = *src_buf_ptr;


            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;
             
                tar_x_col = tar_x;   
                tar_y_row++;
            }
        }


        return;
}



void BlitBufAbsolute(
        unsigned int d,
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,  
        int src_y,  
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom                     /* 0.1 to 1.0 supported. */
)
{
	switch(d)
	{
	  case 32:
	    BlitBufAbsolute32(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom                     /* 0.1 to 1.0 supported. */
	    );
	    break;

          case 24:
            BlitBufAbsolute32(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom                     /* 0.1 to 1.0 supported. */
            );
            break;

          case 16:
            BlitBufAbsolute16(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom                     /* 0.1 to 1.0 supported. */
            );
            break;

          case 8:
            BlitBufAbsolute8(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom                     /* 0.1 to 1.0 supported. */
 	    );
	    break;
	}

	return;
}



/*
 *                       Additive Blitting
 */
void BlitBufAdditive8(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom   
)
{
        int tar_x_col, tar_y_row;
        int src_x_col, src_y_row;
        unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        int tar_width_bytes;   /* tar_width * BYTES_PER_PIXEL8 */
        int src_width_bytes;   /* src_width * BYTES_PER_PIXEL8 */

        int src_width_check;   /* src_x + src_copy_width */
        int src_height_check;  /* src_y + src_copy_height */

        u_int8_t *tar_buf_ptr;
        u_int8_t *src_buf_ptr;
 
        /* Needed for additive stuff. */
/* YES these are 32 bits for 8 bit blitting! */
        u_int32_t r, g, b;
        u_int32_t c1, c2;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) || 
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||  
           (src_height < 1) ||  
           (src_copy_width < 1) ||  
           (src_copy_height < 1) ||  
           (zoom <= 0)
        )
            return;

        /* Sanitize zoom to be 1 or less. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)  
        )
            return;


        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL8;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL8;

        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);

        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom); 
        y_pixels_inc = (unsigned int)(256 / zoom);

        /* Sanitize target values. */  
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom)); 
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }

        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;

        /* Set up starting numbers. */
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;


        /* ********************************************************* */

        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }

                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int8_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL8)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int8_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +
                (tar_x_col * BYTES_PER_PIXEL8)
            ]);


/* XSW_BUFOPT1 */

            c1 = (u_int32_t)*tar_buf_ptr;
            c2 = (u_int32_t)*src_buf_ptr;

            r = (c1 & 0x000000E0) + (c2 & 0x000000E0);
            g = (c1 & 0x0000001C) + (c2 & 0x0000001C);
            b = (c1 & 0x00000003) + (c2 & 0x00000003);

            *tar_buf_ptr = (u_int8_t)(
                MIN(r, 0x000000E0) | MIN(g, 0x0000001C) |
                MIN(b, 0x00000003)
            );

/* XSW_BUFOPT1 */

            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;

                tar_x_col = tar_x;
                tar_y_row++;
            }
        }


        return;
}


void BlitBufAdditive16(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom
)
{
        int tar_x_col, tar_y_row;
        int src_x_col, src_y_row;

        unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        int tar_width_bytes;	/* tar_width * BYTES_PER_PIXEL16 */
        int src_width_bytes;	/* src_width * BYTES_PER_PIXEL16 */

        int src_width_check;	/* src_x + src_copy_width */
        int src_height_check;	/* src_y + src_copy_height */

        u_int16_t *tar_buf_ptr;
        u_int16_t *src_buf_ptr;

        /* Needed for additive stuff. */
/* YES these are 32 bits for 16 bit blitting! */
        u_int32_t r, g, b;

        u_int32_t c1, c2;


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
           (zoom <= 0)
        )
            return;

        /* Sanitize zoom to be 1 or less. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;



        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL16;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL16;

        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);

        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom);
            
        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }

        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;

        /* Set up starting numbers. */   
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;


        /* ********************************************************* */

        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;
                
                    tar_x_col = tar_x;
                    tar_y_row++;
                }

                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int16_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL16)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int16_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +  
                (tar_x_col * BYTES_PER_PIXEL16)  
            ]);


/* XSW_BUFOPT1 */

            c1 = (u_int32_t)*tar_buf_ptr;
            c2 = (u_int32_t)*src_buf_ptr;

            r = (c1 & 0x0000F800) + (c2 & 0x0000F800);
            g = (c1 & 0x000007E0) + (c2 & 0x000007E0);
            b = (c1 & 0x0000001F) + (c2 & 0x0000001F);

            *tar_buf_ptr = (u_int16_t)(
		MIN(r, 0x0000F800) | MIN(g, 0x000007E0) |
                MIN(b, 0x0000001F)
	    );

/* XSW_BUFOPT1 */

            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {   
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;

                tar_x_col = tar_x;
                tar_y_row++;
            }
        }


        return;
}



void BlitBufAdditive32(
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
	double zoom
)
{
        int tar_x_col, tar_y_row;
        int src_x_col, src_y_row;

        unsigned int x_pixels_inc, y_pixels_inc;    /* For zooming. */

        int tar_width_bytes;	/* tar_width * BYTES_PER_PIXEL32 */
        int src_width_bytes;	/* src_width * BYTES_PER_PIXEL32 */

        int src_width_check;	/* src_x + src_copy_width */
        int src_height_check;	/* src_y + src_copy_height */

        u_int32_t *tar_buf_ptr;
        u_int32_t *src_buf_ptr;

	/* Needed for additive stuff. */
#ifdef XSW_BUFOPT1
	u_int32_t r, g, b;
#endif

#ifdef XSW_BUFOPT2
	u_int32_t r, t;
#endif


        /* Error checks. */
        if((tar_buf == NULL) ||
           (src_buf == NULL) ||
           (tar_width < 1) ||
           (tar_height < 1) ||
           (src_width < 1) ||
           (src_height < 1) ||
           (src_copy_width < 1) ||
           (src_copy_height < 1) ||
	   (zoom <= 0)
        )
            return;


        /* Sanitize zoom to be 1 or less. */
        zoom = (zoom > 1) ? 1 : zoom;

        /* Source totally not visable on target? */
        if(((tar_x + (int)((double)src_copy_width * zoom)) < 0) ||
           ((tar_y + (int)((double)src_copy_height * zoom)) < 0) ||
           (tar_x >= (int)tar_width) ||
           (tar_y >= (int)tar_height) ||
           ((src_x + (int)src_copy_width) < 0) ||
           ((src_y + (int)src_copy_height) < 0) ||
           (src_x >= (int)src_width) ||
           (src_y >= (int)src_height)
        )
            return;


        /* ************************************************************* */

        /* Precalculate byte sizes (before converting). */
        tar_width_bytes = (int)tar_width * BYTES_PER_PIXEL32;
        src_width_bytes = (int)src_width * BYTES_PER_PIXEL32;

        /* Convert source units to 256 (for zooming). */
        src_x *= 256;
        src_y *= 256;
        src_copy_width = (int)src_copy_width * 256;
        src_copy_height = (int)src_copy_height * 256;
        src_width = (int)src_width * 256;
        src_height = (int)src_height * 256;

        /* Precalculate source checks. */
        src_width_check = (int)(src_x + (int)src_copy_width);
        src_height_check = (int)(src_y + (int)src_copy_height);

        /* Calculate source increments for zooming affect. */
        x_pixels_inc = (unsigned int)(256 / zoom);
        y_pixels_inc = (unsigned int)(256 / zoom);
            
        /* Sanitize target values. */
        if(tar_x < 0)
        {
            src_x += (tar_x * -256 * (1 / zoom));
            tar_x = 0;
        }
        if(tar_y < 0)
        {
            src_y += (tar_y * -256 * (1 / zoom));
            tar_y = 0;
        }

        if(src_x < 0) src_x = 0;
        if(src_y < 0) src_y = 0;

        if(src_width_check > (int)src_width)
            src_width_check = src_width;
        if(src_height_check > (int)src_height)
            src_height_check = src_height;

        /* Set up starting numbers. */   
        tar_x_col = tar_x;
        tar_y_row = tar_y;
        src_x_col = src_x;
        src_y_row = src_y;


        /* ********************************************************* */

        /* Begin blitting src_buf to tar_buf. */
        while(src_y_row < src_height_check)
        {
            /* Skip if off the screen. */
            if((tar_x_col >= (int)tar_width) ||
               (tar_y_row >= (int)tar_height) ||
               (src_x_col >= (int)src_width) ||
               (src_y_row >= (int)src_height)
            )
            {
                /* Increment x colums. */
                src_x_col += (int)x_pixels_inc;
                tar_x_col++;

                /* Increment rows. */
                if(src_x_col >= src_width_check)
                {
                    src_x_col = src_x;
                    src_y_row += (int)y_pixels_inc;

                    tar_x_col = tar_x;
                    tar_y_row++;
                }

                continue;
            }


            /* Get pointer in source buffer. */
            src_buf_ptr = (u_int32_t *)(&src_buf[
                ((src_y_row >> 8) * src_width_bytes) +
                ((src_x_col >> 8) * BYTES_PER_PIXEL32)
            ]);

            /* Get pointer in target buffer. */
            tar_buf_ptr = (u_int32_t *)(&tar_buf[
                (tar_y_row * tar_width_bytes) +
                (tar_x_col * BYTES_PER_PIXEL32)
            ]);


/* Additive Merge - Optimization Style 1 */
#ifdef XSW_BUFOPT1
	    c1 = (u_int32_t)*tar_buf_ptr;
            c2 = (u_int32_t)*src_buf_ptr;

            r = (c1 & 0x00FF0000) + (c2 & 0x00FF0000);
            g = (c1 & 0x0000FF00) + (c2 & 0x0000FF00);
            b = (c1 & 0x000000FF) + (c2 & 0x000000FF);

            *tar_buf_ptr = MIN(r, 0x00FF0000) | MIN(g, 0x0000FF00) |
                MIN(b, 0x000000FF);

#endif	/* XSW_BUFOPT1 */


/* Additive Merge - Optimization Style 2 */
#ifdef XSW_BUFOPT2
	    c1 = (u_int32_t)*tar_buf_ptr;
	    c2 = (u_int32_t)*src_buf_ptr;

	    t = c1 & 0x00FF0000;
            r = (t + (c2 & 0x00FF0000)) & 0x00FF0000;
            if(r < t)
                c = 0x00FF0000;
            else
                c = r;

            t = c1 & 0x0000FF00;
            r = (t + (c2 & 0x0000FF00)) & 0x0000FF00;
            if(r < t)
                c |= 0x0000FF00;
            else
                c |= r;

            t = c1 & 0x000000FF;
            r = (t + (c2 & 0x000000FF)) & 0x000000FF;
            if(r < t)
                c |= 0x000000FF;
            else
                c |= r;

            *tar_buf_ptr = (u_int32_t)c;
#endif	/* XSW_BUFOPT2 */


            /* Increment x colums. */
            src_x_col += (int)x_pixels_inc;
            tar_x_col++;

            /* Increment rows. */
            if(src_x_col >= src_width_check)
            {   
                src_x_col = src_x;
                src_y_row += (int)y_pixels_inc;

                tar_x_col = tar_x;
                tar_y_row++;
            }
        }


        return;
}



void BlitBufAdditive(
        unsigned int d,
        char *tar_buf,
        char *src_buf,
        int tar_x,
        int tar_y,
        unsigned int tar_width,
        unsigned int tar_height,
        int src_x,
        int src_y,
        unsigned int src_width,
        unsigned int src_height,
        unsigned int src_copy_width,
        unsigned int src_copy_height,
        double zoom                     /* 0.1 to 1.0 supported. */
)
{
	switch(d)
	{
	  case 32:
	    BlitBufAdditive32(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,   
                zoom
	    );
	    break;

          case 24:
            BlitBufAdditive32(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom
            );
            break;

          case 16:
            BlitBufAdditive16(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width,
                src_copy_height,
                zoom
            );
            break;

          case 8:
            BlitBufAdditive8(
                tar_buf,
                src_buf,
                tar_x,
                tar_y,
                tar_width,
                tar_height,
                src_x,
                src_y,
                src_width,
                src_height,
                src_copy_width, 
                src_copy_height, 
                zoom
            );
            break;
	}

	return;
}



/*
 *	Simple solid buffer blitting
 */
void BlitBufSolid8(
        char *tar_buf,
        unsigned int tar_width,
        unsigned int tar_height,
        WColorStruct color
)
{
        register int x, buf_len;
        register u_int8_t value;
        register u_int8_t *tar_buf_ptr;


        /* Error checks. */
        if( (tar_buf == NULL) ||   
            (tar_width == 0) ||
            (tar_height == 0)
        )
            return;


        /* Get starting buffer position. */
        tar_buf_ptr = (u_int8_t *)tar_buf;

        /* Get length of buffer in bytes. */
        buf_len = (int)tar_width * (int)tar_height * BYTES_PER_PIXEL8;

        /* Set `pixel' value. */
        value = (u_int8_t)PACK8TO8(color.r, color.g, color.b);

        /* Begin setting buffer. */
        for(x = 0; x < buf_len; x += BYTES_PER_PIXEL8)
            *tar_buf_ptr++ = value;


        return;
}


void BlitBufSolid16(
        char *tar_buf,
        unsigned int tar_width,
        unsigned int tar_height,
        WColorStruct color
)
{
        register int x, buf_len;
        register u_int16_t value;
        register u_int16_t *tar_buf_ptr;
 
        
        /* Error checks. */
        if( (tar_buf == NULL) ||
            (tar_width == 0) ||
            (tar_height == 0)
        )
            return;


        /* Get starting buffer position. */
        tar_buf_ptr = (u_int16_t *)tar_buf;
        
        /* Get length of buffer in bytes. */
        buf_len = (int)tar_width * (int)tar_height * BYTES_PER_PIXEL16;
        
        /* Set `pixel' value. */ 
        value = (u_int16_t)PACK8TO16(color.r, color.g, color.b);
        
        /* Begin setting buffer. */
        for(x = 0; x < buf_len; x += BYTES_PER_PIXEL16)
            *tar_buf_ptr++ = value;

            
        return;
}


void BlitBufSolid32(
        char *tar_buf,
        unsigned int tar_width,
        unsigned int tar_height,
        WColorStruct color
)
{
	register int x, buf_len;
	register u_int32_t value;
	register u_int32_t *tar_buf_ptr;


	/* Error checks. */
	if( (tar_buf == NULL) ||
	    (tar_width == 0) ||
	    (tar_height == 0)
	)
	    return;


	/* Get starting buffer position. */
	tar_buf_ptr = (u_int32_t *)tar_buf;

	/* Get length of buffer in bytes. */
	buf_len = (int)tar_width * (int)tar_height * BYTES_PER_PIXEL32;

	/* Set `pixel' value. */
	value = (u_int32_t)PACK8TO32(color.a, color.r, color.g, color.b);

	/* Begin setting buffer. */
	for(x = 0; x < buf_len; x += BYTES_PER_PIXEL32)
	    *tar_buf_ptr++ = value;


	return;
}



void BlitBufSolid(
        unsigned int d,
        char *tar_buf,
        unsigned int tar_width,
        unsigned int tar_height,
        WColorStruct color
)
{
	switch(d)
	{
	  case 32:
	    BlitBufSolid32(
                tar_buf,
                tar_width,
                tar_height,
                color
	    );
	    break;

          case 24:
            BlitBufSolid32(
                tar_buf,
                tar_width,
                tar_height,
                color
            );
            break;

          case 16:
            BlitBufSolid16(
                tar_buf,
                tar_width,
                tar_height,
                color
            );
            break;

          case 8:
            BlitBufSolid8(
                tar_buf,
                tar_width,
                tar_height,
                color
            );
            break;
	}

	return;
}
