/*
                         X Joystick Caliberator

	Functions:

	void XJSCalHelp()
	void XJSCalVersion()

	int XJSCalFBOK(char *path)
	int XJSCalFBCancel(char *path)

	void XJSCalUnfocusAll()
	void XJSCalShutdown()

	void XJSCalDrawButtons()
	void XJSCalDrawSwitches()
	void XJSCalDrawReadout()
	void XJSCalDrawCoords()
	void XJSCalDrawAll()

	void XJSCalDoCalib()

	int main(int argc, char *argv[])

	---

 */

#include "xjscal.h"

#include "images/crosshair.xpm"
#include "images/calib_text.xpm"
#include "images/exit_button0.xpm"
#include "images/exit_button1.xpm"
#include "images/jscoords.xpm"
#include "images/mode_switch0.xpm"
#include "images/mode_switch1.xpm"
#include "images/nojs_text.xpm"
#include "images/num_switch0.xpm"
#include "images/num_switch1.xpm"
#include "images/num_switch2.xpm"
#include "images/num_switch3.xpm"
#include "images/readout.xpm"
#include "images/save_button0.xpm"
#include "images/save_button1.xpm"
#include "images/toplevel_bkg.xpm"

#include "js_icon.h"


#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))



/*
 *	Prints help.
 */
void XJSCalHelp()
{
	printf(XJSCAL_USAGE);

	return;
}


/*
 *	Prints version information.
 */
void XJSCalVersion()
{
	printf(
 "%s Version %s\n",
	    XJSCAL_NAME, XJSCAL_VERSION
	);
	printf(
 "Copyright (C) 1999 WolfPack Development Team.\n"
	);

	return;
}



/*
 *	File browser callbacks.
 */
int XJSCalFBOK(char *path)
{
	int status;
	char stringa[PATH_MAX + NAME_MAX + 80];


	status = JSWriteCaliberationFile(path);
	if(status == -1)
	{
	    sprintf(stringa,
		"Unable to save file:\n\n    %s",
		path
	    );
            XJSCalUnfocusAll();
	    printdw(&dialog, stringa);
	}
	else
	{
            sprintf(stringa,
                "Joystick caliberation saved to file:\n\n    %s",
                path
            );
            XJSCalUnfocusAll();
            printdw(&dialog, stringa);
	}

	return(0);
}


int XJSCalFBCancel(char *path) 
{
	return(0);
}


/*
 *	Unfocus all of this program's toplevel windows
 *	except those of widgets.
 */
void XJSCalUnfocusAll()
{
	xjscal_win.focused = False;

	return;
}


/*
 *	Deallocates all allocated resources.
 */
void XJSCalShutdown()
{
	DialogWinDestroy(&dialog);
	FBrowserDestroy(&fb);


	/* Free the pixmaps. */
	OSWDestroyPixmap(&js_icon_pm);

	OSWDestroyPixmap(&xjscal_win.toplevel_pm);
        OSWDestroyPixmap(&xjscal_win.exit_btn0_pm);
        OSWDestroyPixmap(&xjscal_win.exit_btn1_pm);
        OSWDestroyPixmap(&xjscal_win.mode_switch0_pm);
        OSWDestroyPixmap(&xjscal_win.mode_switch1_pm);
        OSWDestroyPixmap(&xjscal_win.num_switch0_pm);
        OSWDestroyPixmap(&xjscal_win.num_switch1_pm);
        OSWDestroyPixmap(&xjscal_win.num_switch2_pm);
        OSWDestroyPixmap(&xjscal_win.num_switch3_pm);
        OSWDestroyPixmap(&xjscal_win.readout_pm);
        OSWDestroyPixmap(&xjscal_win.save_btn0_pm);
        OSWDestroyPixmap(&xjscal_win.save_btn1_pm);

	OSWDestroySharedImage(
	    &xjscal_win.coord_readout_buf_img
	);
	OSWDestroyImage(&xjscal_win.coord_readout_img);
	OSWDestroyImage(&xjscal_win.crosshair_img);

        OSWDestroyImage(&xjscal_win.calib_text_img);
        OSWDestroyImage(&xjscal_win.nojs_text_img);


        /* Color pixels. */
        OSWDestroyPixel(&normal_text_pixel);
        OSWDestroyPixel(&bold_text_pixel);
        OSWDestroyPixel(&readout_text_pixel);


	/* Destroy the windows. */
        OSWDestroyWindow(&xjscal_win.exit_btn);
        OSWDestroyWindow(&xjscal_win.save_btn);
        OSWDestroyWindow(&xjscal_win.js_num_switch);
        OSWDestroyWindow(&xjscal_win.mode_switch);
        OSWDestroyWindow(&xjscal_win.btn_readout);
        OSWDestroyWindow(&xjscal_win.coord_readout);
	OSWDestroyWindow(&xjscal_win.toplevel);


        /*   Free widget globals after all client allocated widgets
         *   have been deallocated.
         */
        WidgetDestroyGlobals();

	/* Disconnect from the GUI. */
	OSWGUIDisconnect();

        /* Close all joystick devices. */
        JSClose(&jscal_rec[0]);
        JSClose(&jscal_rec[1]);
        JSClose(&jscal_rec[2]);
        JSClose(&jscal_rec[3]);


        return;
}



/*
 *	Redraw all buttons.
 */
void XJSCalDrawButtons()
{
	/* Save button. */
        OSWPutBufferToWindow(
            xjscal_win.save_btn,
            (xjscal_win.save_btn_state) ?
		xjscal_win.save_btn1_pm : xjscal_win.save_btn0_pm
        );

	/* Exit button. */
        OSWPutBufferToWindow(
            xjscal_win.exit_btn,
            (xjscal_win.exit_btn_state) ?
                xjscal_win.exit_btn1_pm : xjscal_win.exit_btn0_pm
        );


	return;
}



/*
 *	Redraws all switches.
 */
void XJSCalDrawSwitches()
{
	/* Mode switch. */
	switch(xjscal_win.mode_switch_state)
	{
          case MODE_SWITCH_CALIB:
	    OSWPutBufferToWindow(
		xjscal_win.mode_switch, xjscal_win.mode_switch1_pm
	    );
	    break;
          default:
            OSWPutBufferToWindow(
                xjscal_win.mode_switch, xjscal_win.mode_switch0_pm
            );
            break;
	}

        /* Joystick number switch. */
	switch(xjscal_win.sel_js_num)
	{
          case 3:
            OSWPutBufferToWindow(
                xjscal_win.js_num_switch, xjscal_win.num_switch3_pm
            );
            break;

	  case 2:
            OSWPutBufferToWindow(
                xjscal_win.js_num_switch, xjscal_win.num_switch2_pm
            );
            break;

	  case 1:
            OSWPutBufferToWindow(
                xjscal_win.js_num_switch, xjscal_win.num_switch1_pm
            );
            break;

	  default:
            OSWPutBufferToWindow(
                xjscal_win.js_num_switch, xjscal_win.num_switch0_pm
	    );
            break;
	}


	return;
}


/*
 *      Redraws buttons readout window.
 */
void XJSCalDrawReadout()
{
	static int js_num;
	static win_attr_t wattr;
	static char stringa[256];


	/* Get window attributes. */
	OSWGetWindowAttributes(xjscal_win.btn_readout, &wattr);

	/* Reset buffer. */
	OSWCopyDrawablesCoord(
	    xjscal_win.tmp_buf_pm,
	    xjscal_win.readout_pm,
	    0, 0,
	    wattr.width, wattr.height,
	    0, 0
	);
	OSWSetFgPix(readout_text_pixel);


	/* Get and sanitize js_num. */
	js_num = xjscal_win.sel_js_num;
	if(js_num >= MAX_JOYSTICKS)
	    js_num = MAX_JOYSTICKS - 1;
	if(js_num < 0)
	    js_num = 0;


	/*   When in caliberation mode, min and max values
	 *   are displayed.
	 */
	if(xjscal_win.mode_switch_state == MODE_SWITCH_CALIB)
	{
	    sprintf(stringa, "X MIN: %i",
		jscal_rec[js_num].x_min
	    );
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 31, stringa);

            sprintf(stringa, "X MAX: %i",
                jscal_rec[js_num].x_max
	    );
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 62, stringa);


            sprintf(stringa, "Y MIN: %i",
                jscal_rec[js_num].y_min
	    );
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 91, stringa);

            sprintf(stringa, "Y MAX: %i",
                jscal_rec[js_num].y_max
	    );
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 120, stringa);

            sprintf(stringa, "NZ: %i",
                jscal_rec[js_num].nullzone
	    );
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 149, stringa);
	}
	else
	{
	    /* Button 1. */
	    if(jscal_rec[js_num].buttons & (1 << 0))
                sprintf(stringa, "B1: ON");
	    else
	        sprintf(stringa, "B1: OFF");
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 31, stringa);

            /* Button 2. */
            if(jscal_rec[js_num].buttons & (1 << 1))
                sprintf(stringa, "B2: ON");
            else
                sprintf(stringa, "B2: OFF");
	    OSWDrawString(xjscal_win.tmp_buf_pm, 32, 62, stringa);

            /* Button 3. */
            if(jscal_rec[js_num].buttons & (1 << 2))
                sprintf(stringa, "B3: ON");
            else
                sprintf(stringa, "B3: OFF");
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 91, stringa);

            /* Button 4. */
            if(jscal_rec[js_num].buttons & (1 << 3))
                sprintf(stringa, "B4: ON");
            else
                sprintf(stringa, "B4: OFF");
	    OSWDrawString(xjscal_win.tmp_buf_pm, 32, 120, stringa);

            /* Button 5. */
            if(jscal_rec[js_num].buttons & (1 << 4))
                sprintf(stringa, "B5: ON");
            else
                sprintf(stringa, "B5: OFF");
            OSWDrawString(xjscal_win.tmp_buf_pm, 32, 149, stringa);
	}


        /* Copy buffer. */
	OSWPutBufferToWindow(
	    xjscal_win.btn_readout,
	    xjscal_win.tmp_buf_pm
	);


	return;
}



/*
 *	Redraws joystick coordinates window.
 */
void XJSCalDrawCoords()
{
	static int js_num;
	static double nz;
	static double js_x, js_x_coeff, js_y, js_y_coeff;
	static double js_x_min, js_x_max;
        static double js_y_min, js_y_max;
	static double js_x_scope, js_y_scope;
	static WColorStruct color;


	/* Get joystick number. */
	js_num = xjscal_win.sel_js_num;
	if(js_num >= MAX_JOYSTICKS)
	    js_num = MAX_JOYSTICKS - 1;
	if(js_num < 0)
	    js_num = 0;


	/* Clear coordinates readout buffer image. */
	BlitBufAbsolute(
	    osw_gui[0].depth,
            xjscal_win.coord_readout_buf_img->data,	/* Target. */
            xjscal_win.coord_readout_img->data,		/* Source. */
	    0, 0,
            xjscal_win.coord_readout_buf_img->width,
	    xjscal_win.coord_readout_buf_img->height,
            0, 0,
            xjscal_win.coord_readout_img->width,
	    xjscal_win.coord_readout_img->height,
            xjscal_win.coord_readout_img->width,
	    xjscal_win.coord_readout_img->height,
            1.0			/* Zoom. */
	);


	/* Are we in caliberation mode? */
	if(xjscal_win.mode_switch_state == MODE_SWITCH_CALIB)
	{
            /* Caliberation instruction message. */
            BlitBufNormal(
                osw_gui[0].depth,
                xjscal_win.coord_readout_buf_img->data,	/* Target. */
                xjscal_win.calib_text_img->data,	/* Source. */
                18, 36,
                xjscal_win.coord_readout_buf_img->width,
                xjscal_win.coord_readout_buf_img->height,
                0, 0,
                xjscal_win.calib_text_img->width,
                xjscal_win.calib_text_img->height,
                xjscal_win.calib_text_img->width, 
                xjscal_win.calib_text_img->height,
                1.0,                        /* Zoom. */
                1.0                         /* Visibility. */
            );
	}
	else
	{
	    /* See if joystick exists. */
	    if(!jscal_rec[js_num].exists)
	    {
		/* Non-existant joystick. */
                BlitBufNormal(
                    osw_gui[0].depth,
                    xjscal_win.coord_readout_buf_img->data,     /* Target. */
                    xjscal_win.nojs_text_img->data,		/* Source. */
                    18, 36,
                    xjscal_win.coord_readout_buf_img->width,
                    xjscal_win.coord_readout_buf_img->height,
                    0, 0,
                    xjscal_win.nojs_text_img->width,
                    xjscal_win.nojs_text_img->height,
                    xjscal_win.nojs_text_img->width,
                    xjscal_win.nojs_text_img->height,
                    1.0,                        /* Zoom. */
                    1.0                         /* Visibility. */
                );
	    }
	    else
	    {
	        /* Get joystick position. */
		JSRead(&jscal_rec[js_num]);

	        /* Get joystick axis min and max values. */
                js_x_min = jscal_rec[js_num].x_min;
                js_x_max = jscal_rec[js_num].x_max;
                js_y_min = jscal_rec[js_num].y_min;
                js_y_max = jscal_rec[js_num].y_max;

		js_x_scope = jscal_rec[js_num].x_scope;
		js_y_scope = jscal_rec[js_num].y_scope;

		/* Get nullzone radius. */
	        nz = jscal_rec[js_num].nullzone;


	        /* Calculate and draw nullzone circle. */
	        nz = (nz / MAX(js_x_scope, 1)) * 160;
	        js_x = 93;
	        js_y = 94;

	        color.a = 0x00;
	        color.r = 0xe0;
                color.g = 0x38;
                color.b = 0x31;

	        BlitBufCircle(
		    osw_gui[0].depth,
		    xjscal_win.coord_readout_buf_img->data,
		    js_x, js_y,
                    xjscal_win.coord_readout_buf_img->width,
		    xjscal_win.coord_readout_buf_img->height,
		    1,
		    nz,
		    color
	        );


                /* Calculate and draw crosshairs. */
		js_x_coeff = jscal_rec[js_num].x_coeff + 1;
                js_y_coeff = jscal_rec[js_num].y_coeff + 1;

		js_x = (js_x_coeff * 80) + 12;
                js_y = (js_y_coeff * 80) + 15;

		if(js_x < 12) js_x = 12;
		if(js_y < 15) js_y = 15;

	        BlitBufNormal(
		    osw_gui[0].depth,
		    xjscal_win.coord_readout_buf_img->data,	/* Target. */
		    xjscal_win.crosshair_img->data,		/* Source. */
		    js_x - (xjscal_win.crosshair_img->width / 2),
		    js_y - (xjscal_win.crosshair_img->height / 2),
		    xjscal_win.coord_readout_buf_img->width,
		    xjscal_win.coord_readout_buf_img->height,
		    0, 0,
		    xjscal_win.crosshair_img->width,
		    xjscal_win.crosshair_img->height,
		    xjscal_win.crosshair_img->width,
		    xjscal_win.crosshair_img->height,
		    1.0,			/* Zoom. */
		    1.0				/* Visibility. */
	        );
	    }
	}

	OSWPutSharedImageToDrawable(
	    xjscal_win.coord_readout_buf_img,
	    xjscal_win.coord_readout
	);



	return;
}



/*
 *	Redraw all.
 */
void XJSCalDrawAll()
{
	/* Redraw toplevel. */
	OSWPutBufferToWindow(
	    xjscal_win.toplevel,
	    xjscal_win.toplevel_pm
	);
	
	/* Redraw buttons. */
	XJSCalDrawButtons();

	/* Redraw switches. */
	XJSCalDrawSwitches();

	/* Redraw readout. */
	XJSCalDrawReadout();


	return;
}



/*
 *	Does the caliberation by updating the min and max of the currently
 *	selected joystick record.   This function should be called on
 *	every loop when in caliberation mode.
 */
void XJSCalDoCalib()
{
	static int i;


	/* Get number of joystick we are caliberating. */
	i = xjscal_win.sel_js_num;
	if(i >= MAX_JOYSTICKS)
	    i = MAX_JOYSTICKS - 1;
	if(i < 0)
	    i = 0;


	/* See if that joystick is valid. */
	if(!jscal_rec[i].exists)
	    return;

	/* Fetch joystick coordinates. */
	JSRead(&jscal_rec[i]);


	/*   If stick has gone farther than last min or max,
	 *   then set new min or max.
	 */
	if(jscal_rec[i].x < jscal_rec[i].x_min)
	    jscal_rec[i].x_min = jscal_rec[i].x;

	if(jscal_rec[i].x > jscal_rec[i].x_max)
	    jscal_rec[i].x_max = jscal_rec[i].x;


        if(jscal_rec[i].y < jscal_rec[i].y_min)
            jscal_rec[i].y_min = jscal_rec[i].y;

        if(jscal_rec[i].y > jscal_rec[i].y_max)
            jscal_rec[i].y_max = jscal_rec[i].y;

	/* Set center unconditionally. */
	jscal_rec[i].x_center = jscal_rec[i].x;
	jscal_rec[i].y_center = jscal_rec[i].y;

	/* Update scopes. */
	jscal_rec[i].x_scope = jscal_rec[i].x_max - jscal_rec[i].x_min;
        jscal_rec[i].y_scope = jscal_rec[i].y_max - jscal_rec[i].y_min;


	return;
}


/*
 *	Reset all global timming schduals.
 */
void XJSCalResetTimmers()
{
	next.redraw_buttons = 0;
	next.redraw_coordinates = 0;

	return;
}



int main(int argc, char *argv[])
{
        int x;
	char *strptr;
	char stringa[256];
	char cwd[PATH_MAX];

	event_t event;

	image_t *image;
	pixmap_t pixmap;

	struct stat stat_buf;
	time_t prev_millitime;

        unsigned int xjscal_width = 320; 
        unsigned int xjscal_height = 240;


	getcwd(cwd, PATH_MAX);
	cwd[PATH_MAX - 1] = '\0';


        /* ******************************************************* */
        /* Set global variables to default values. */
	runlevel = 1;

	XJSCalResetTimmers();

	strncpy(fname.js_device, DEV_JOYSTICK_0, PATH_MAX + NAME_MAX);
	fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

	xjscal_win.sel_js_num = 0;		/* 0 to MAX_JOYSTICKS - 1. */
	xjscal_win.mode_switch_state = MODE_SWITCH_TESTING;

	/* Set joystick structure to defaults. */
	for(x = 0; x < MAX_JOYSTICKS; x++)
	{
	    jscal_rec[x].exists = 0;
	    jscal_rec[x].x_min = DEFAULT_X_AXIS_MIN_VALUE;
            jscal_rec[x].x_max = DEFAULT_X_AXIS_MAX_VALUE;
            jscal_rec[x].y_min = DEFAULT_Y_AXIS_MIN_VALUE;
            jscal_rec[x].y_max = DEFAULT_Y_AXIS_MAX_VALUE;
	    jscal_rec[x].nullzone = DEFAULT_NULLZONE_VALUE;
	}

	xjscal_win.save_btn_state = False;
	xjscal_win.exit_btn_state = False;

	strptr = getenv("HOME");
	sprintf(fname.js_calib, "%s/%s",
	    (strptr == NULL) ? "" : strptr, JS_CALIBRATION_FILE
	);


        /* ********************************************* */
        /* ***     Parse Command Line Parameters     *** */
        for(x = 1; x < argc; x++)
	{
	    /* Help. */
            if(strpfx(argv[x], "--h") ||
               strpfx(argv[x], "-h") ||
               strpfx(argv[x], "-?")
	    )
            {
                XJSCalHelp();
                return(0);
            }
	    /* Version. */
            else if(strpfx(argv[x], "--ver") ||
                    strpfx(argv[x], "-ver")
	    )
            {
                XJSCalVersion();
                return(0);
            }
	    /* Device. */
            else if(strpfx(argv[x], "--d") ||
                    strpfx(argv[x], "-d")
	    )
	    {
		x++;
		if(x < argc)
		{
		    strncpy(
			fname.js_device,
			argv[x],
			PATH_MAX + NAME_MAX
		    );
		    fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';
		}
		else
		{
		    fprintf(stderr,
			"%s: Option `%s' requires an argument.\n",
			argv[0],
			argv[x - 1]
		    );
		    return(0);
		}
	    }
	    /* Joystick caliberation file. */
            else if(strpfx(argv[x], "--f") ||
                    strpfx(argv[x], "-f")
	    )
            {
                x++;
                if(x < argc)
                {
                    strncpy(
                        fname.js_calib,    
                        argv[x],
                        PATH_MAX + NAME_MAX
                    );
                    fname.js_calib[PATH_MAX + NAME_MAX - 1] = '\0';
                }   
                else
                {
                    fprintf(stderr,
                        "%s: Option `%s' requires an argument.\n",
                        argv[0],
                        argv[x - 1]
                    );
                    return(0);
                }
            }
        }



	/* See which device node we need to switch to at startup. */
	if(!strcmp(fname.js_device, DEV_JOYSTICK_0))
	{
	    xjscal_win.sel_js_num = 0;
	}
        else if(!strcmp(fname.js_device, DEV_JOYSTICK_1))
        {
            xjscal_win.sel_js_num = 1;
        }
        else if(!strcmp(fname.js_device, DEV_JOYSTICK_2))
        {
            xjscal_win.sel_js_num = 2;
        }
        else if(!strcmp(fname.js_device, DEV_JOYSTICK_3))
        {
            xjscal_win.sel_js_num = 3;
        }


        /* ****************************************************** */

        /*   If the caliberation file does not exist, start up in
	 *   caliberation mode.
         */
        if(stat(fname.js_calib, &stat_buf))
        {
            xjscal_win.mode_switch_state = MODE_SWITCH_CALIB;
	    x = xjscal_win.sel_js_num;

            /* Reset the joystick records before caliberating. */
            jscal_rec[x].x_min = MAX_JS_X_AXIS_VALUE;
            jscal_rec[x].x_max = MIN_JS_X_AXIS_VALUE;
            jscal_rec[x].y_min = MAX_JS_Y_AXIS_VALUE;
            jscal_rec[x].y_max = MIN_JS_Y_AXIS_VALUE;
        }


        /* ************************************************* */
        /* ***          Initialize JoyStick              *** */

	/* Read caliberation from file. */
	x = JSReadCaliberationFile(fname.js_calib);

	/* Get which joystick device nodes exist. */
	if(stat(DEV_JOYSTICK_0, &stat_buf))
	    jscal_rec[0].exists = 1;
        if(stat(DEV_JOYSTICK_1, &stat_buf))
            jscal_rec[1].exists = 1;
        if(stat(DEV_JOYSTICK_2, &stat_buf))
            jscal_rec[2].exists = 1;
        if(stat(DEV_JOYSTICK_3, &stat_buf))
            jscal_rec[3].exists = 1;

	/* Does the one we want to start up with exist? */
	if(stat(fname.js_device, &stat_buf))
	{
	    fprintf(stderr,
		"%s: No such device.\n",
		fname.js_device
	    );
	}

	/* Open joystick device. */
	x = xjscal_win.sel_js_num;
        if(JSOpen(fname.js_device, &jscal_rec[x]) < 0)
	{
            fprintf(stderr,
	        "%s: Cannot open device.\n",
		fname.js_device
	    );
	    fprintf(stderr,
	"Is the joystick device already in use by another program?\n"
	    );
	    fprintf(stderr,
	"Is the joystick module loaded? Type 'lsmod' to check.\n"
	    );
            fprintf(stderr,
	"Is your joystick plugged in?\n"
	    );
	    fprintf(stderr,
	"If have a plug and play sound/game card, is it initialized?\n"
 	    );
	}



        /* *********************************************************** */
	/* Connect to GUI and initialize resources. */

        if(OSWGUIConnect(argc, argv))
	{
            XJSCalShutdown();
	    return(-1);
        }

	if(WidgetInitGlobals(argc, argv))
	{
            XJSCalShutdown();
            return(-1);
	}


        /* Get color pixels. */
	OSWLoadPixelCLSP(&normal_text_pixel, NORMAL_TEXT_COLOR_SPEC);
	OSWLoadPixelCLSP(&bold_text_pixel, BOLD_TEXT_COLOR_SPEC);
	OSWLoadPixelCLSP(&readout_text_pixel, READOUT_TEXT_COLOR_SPEC);


        /* ********************************************************* */
        /* Initialize windows and load images. */

	image = WidgetLoadImageFromXpmData(crosshair_xpm);
	xjscal_win.crosshair_img = image;

	pixmap = WidgetLoadPixmapFromXpmData(toplevel_bkg_xpm);
	xjscal_win.toplevel_pm = pixmap;

        pixmap = WidgetLoadPixmapFromXpmData(exit_button0_xpm);
        xjscal_win.exit_btn0_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(exit_button1_xpm);
        xjscal_win.exit_btn1_pm = pixmap;

        image = WidgetLoadImageFromXpmData(jscoords_xpm);
        xjscal_win.coord_readout_img = image;

        pixmap = WidgetLoadPixmapFromXpmData(mode_switch0_xpm);
        xjscal_win.mode_switch0_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(mode_switch1_xpm);
        xjscal_win.mode_switch1_pm = pixmap;

        pixmap = WidgetLoadPixmapFromXpmData(num_switch0_xpm);
        xjscal_win.num_switch0_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(num_switch1_xpm);
        xjscal_win.num_switch1_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(num_switch2_xpm); 
        xjscal_win.num_switch2_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(num_switch3_xpm); 
        xjscal_win.num_switch3_pm = pixmap;

        pixmap = WidgetLoadPixmapFromXpmData(readout_xpm); 
        xjscal_win.readout_pm = pixmap;

        pixmap = WidgetLoadPixmapFromXpmData(save_button0_xpm);
        xjscal_win.save_btn0_pm = pixmap;
        pixmap = WidgetLoadPixmapFromXpmData(save_button1_xpm);
        xjscal_win.save_btn1_pm = pixmap; 

	/* Message images. */
        image = WidgetLoadImageFromXpmData(calib_text_xpm);
        xjscal_win.calib_text_img = image;
        image = WidgetLoadImageFromXpmData(nojs_text_xpm);
        xjscal_win.nojs_text_img = image;


        /* General buffer pixmap, must be size of toplevel! */
	if(
	    OSWCreatePixmap(
		&xjscal_win.tmp_buf_pm, xjscal_width, xjscal_height
	    )
	)
	    return(-1);

	/* Joystick coordinates readout image buffer (shared) */
	if(
	    OSWCreateSharedImage(
		&xjscal_win.coord_readout_buf_img,
		190, 177		/* Same size as the window. */
	    )
	)
	{
            XJSCalShutdown();
	    return(-1);
	}

	/* Joystick icon image. */
	image = WidgetLoadImageFromTgaData(js_icon_tga);
	pixmap = OSWCreatePixmapFromImage(image);
	OSWDestroyImage(&image);
        js_icon_pm = pixmap;



	/* Toplevel. */
	if(
	    OSWCreateWindow(
		&xjscal_win.toplevel,
		osw_gui[0].root_win,
                osw_gui[0].def_toplevel_x,
                osw_gui[0].def_toplevel_y,
                xjscal_width,
                xjscal_height
	    )
	)
	{
	    XJSCalShutdown();
	    return(-1);
	}

        sprintf(stringa, "%s %s", XJSCAL_NAME, XJSCAL_VERSION);
        OSWSetWindowWMProperties(
            xjscal_win.toplevel,
            stringa,			/* Title. */
            XJSCAL_NAME,		/* Iconified title. */
            js_icon_pm,			/* Icon. */
            !osw_gui[0].def_geometry_set,	/* Let WM set coordinates? */
	    osw_gui[0].def_toplevel_x,
            osw_gui[0].def_toplevel_y,
            xjscal_width, xjscal_height,
	    xjscal_width, xjscal_height,
            WindowFrameStyleFixed,
            NULL, 0   
        );
	OSWSetWindowBkg(xjscal_win.toplevel, 0, xjscal_win.toplevel_pm);


	/* Joystick coordinates position window. */
        if(
            OSWCreateWindow(
                &xjscal_win.coord_readout,
                xjscal_win.toplevel,
                130, 0, 190, 177
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }

	/* Buttons readout window. */
        if(
            OSWCreateWindow(
                &xjscal_win.btn_readout,
                xjscal_win.toplevel,           
                0, 0, 114, 163 
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }


        /* Mode switch window. */
        if(
            OSWCreateWindow(
                &xjscal_win.mode_switch,
                xjscal_win.toplevel,
                130, 190, 83, 50
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }

        /* Joystick number switch window. */
        if(
            OSWCreateWindow(
                &xjscal_win.js_num_switch,
                xjscal_win.toplevel,
                0, 160, 78, 80
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }
           
        /* Save button. */
        if(
            OSWCreateWindow( 
                &xjscal_win.save_btn,
                xjscal_win.toplevel,
                213, 195, 46, 26
            )
        )  
        {
            XJSCalShutdown();   
            return(-1);  
        }

        /* Exit button. */
        if(
            OSWCreateWindow(
                &xjscal_win.exit_btn,
                xjscal_win.toplevel,
                260, 195, 42, 27
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }


	/* Set events to be watched for each window. */
	OSWSetWindowInput(xjscal_win.toplevel, OSW_EVENTMASK_TOPLEVEL);
	OSWSetWindowInput(xjscal_win.mode_switch, ButtonPressMask);
	OSWSetWindowInput(xjscal_win.save_btn, OSW_EVENTMASK_BUTTON);
	OSWSetWindowInput(xjscal_win.exit_btn, OSW_EVENTMASK_BUTTON);
	OSWSetWindowInput(xjscal_win.mode_switch, OSW_EVENTMASK_BUTTON);
	OSWSetWindowInput(xjscal_win.js_num_switch, OSW_EVENTMASK_BUTTON);


	/* Create dialog window. */
	if(
	    DialogWinInit(
		&dialog,
		osw_gui[0].root_win,
		0, 0,
		NULL
            )
        )
        {
            XJSCalShutdown();
            return(-1);
        }
	

	/* Create file browser. */
	if(
	    FBrowserInit(
		&fb,
                0, 0,
                0, 0,
                cwd,                    /* Initial directory. */
                FB_STYLE_SINGLE_LIST,
                XJSCalFBOK,		/* OK callback function. */
                XJSCalFBCancel		/* Cancel callback function. */
	    )
        )
	{
            XJSCalShutdown();
            return(-1);
        }
	FBrowserSetOpMesg(
	    &fb,
	    "Save Caliberation",
            "Save"
	);


	/* Map Windows. */
        OSWMapRaised(xjscal_win.toplevel);
        OSWMapSubwindows(xjscal_win.toplevel);
	xjscal_win.map_state = True;


        /* ******************************************************* */
	runlevel = 2;
        while(runlevel > 1)
        {
	    /* Update timming. */
	    prev_millitime = MilliTime();
	    if(prev_millitime < cur_millitime)
	    {
		XJSCalResetTimmers();
	    }
	    cur_millitime = prev_millitime;


            /*   Must call WidgetManage() once per loop to let
             *   the widget set do what it needs to do.
             */
            WidgetManage(NULL);


            /* ************************************************ */
            /* Manage GUI events. */
	    while(OSWEventsPending() > 0)
	    {
		OSWWaitNextEvent(&event);

                /*   Let WidgetManage() see this event.
                 *   It is not important if the event is handled or not, so
                 *   the return value is disgarded.
                 */
                WidgetManage(&event);

		/* Manage event for our allocated GUI resources. */
                switch(event.type)
                {
		  /* ************************************************** */
		  case KeyPress:
		    /* Do not handle if not in focus. */
		    if(!xjscal_win.focused)
			break;

                    /* Switch back off keyboard repeat. */
                    OSWKBAutoRepeatOff();

                    /* "X"; Exit. */
                    if(event.xkey.keycode == osw_keycode.alpha_x)
                    {
                        xjscal_win.exit_btn_state = True;
                        XJSCalDrawButtons();   
                    }
		    /* "S"; Save to file. */
		    else if(event.xkey.keycode == osw_keycode.alpha_s)
		    {
                        xjscal_win.save_btn_state = True;
                        XJSCalDrawButtons();
		    }
		    /* "1"; Switch to joystick number 1. */
                    else if(event.xkey.keycode == osw_keycode.num_1)
                    {
                        /* Change switch only in testing mode. */
                        if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
                        {
			    JSClose(&jscal_rec[0]);
                            JSClose(&jscal_rec[1]);
                            JSClose(&jscal_rec[2]);
                            JSClose(&jscal_rec[3]);
                            JSOpen(DEV_JOYSTICK_0, &jscal_rec[0]);

			    strncpy(
				fname.js_device,
				DEV_JOYSTICK_0,
				PATH_MAX + NAME_MAX
			    );
			    fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

			    xjscal_win.sel_js_num = 0;
                            XJSCalDrawSwitches();
			}
                    }
                    /* "2"; Switch to joystick number 2. */
                    else if(event.xkey.keycode == osw_keycode.num_2)
                    {
                        /* Change switch only in testing mode. */
                        if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
                        {
                            JSClose(&jscal_rec[0]);
                            JSClose(&jscal_rec[1]);
                            JSClose(&jscal_rec[2]);
                            JSClose(&jscal_rec[3]);
			    JSOpen(DEV_JOYSTICK_1, &jscal_rec[1]);

                            strncpy(
                                fname.js_device,
                                DEV_JOYSTICK_1,
                                PATH_MAX + NAME_MAX
                            );
                            fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

                            xjscal_win.sel_js_num = 1;
                            XJSCalDrawSwitches();
			}
                    }
                    /* "3"; Switch to joystick number 3. */
                    else if(event.xkey.keycode == osw_keycode.num_3)
                    {
                        /* Change switch only in testing mode. */
                        if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
                        {
                            JSClose(&jscal_rec[0]);
                            JSClose(&jscal_rec[1]);
                            JSClose(&jscal_rec[2]);
                            JSClose(&jscal_rec[3]);
                            JSOpen(DEV_JOYSTICK_2, &jscal_rec[2]);

                            strncpy(
                                fname.js_device,
                                DEV_JOYSTICK_2,
                                PATH_MAX + NAME_MAX
                            );
                            fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

                            xjscal_win.sel_js_num = 2;
                            XJSCalDrawSwitches();
			}
                    }
                    /* "4"; Switch to joystick number 4. */
                    else if(event.xkey.keycode == osw_keycode.num_4)
                    {
                        /* Change switch only in testing mode. */
                        if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
			{
                            JSClose(&jscal_rec[0]);
                            JSClose(&jscal_rec[1]);
                            JSClose(&jscal_rec[2]);
                            JSClose(&jscal_rec[3]);
                            JSOpen(DEV_JOYSTICK_3, &jscal_rec[3]);

                            strncpy(
                                fname.js_device,
                                DEV_JOYSTICK_3,
                                PATH_MAX + NAME_MAX
                            );
                            fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

                            xjscal_win.sel_js_num = 3;
                            XJSCalDrawSwitches();
			}
                    }
                    /* "T": Switch to testing. */
                    else if(event.xkey.keycode == osw_keycode.alpha_t)
                    {
                        xjscal_win.mode_switch_state = MODE_SWITCH_TESTING;
                        XJSCalDrawSwitches();
                    }
                    /* "C": Switch to caliberation. */
                    else if(event.xkey.keycode == osw_keycode.alpha_c)
                    {
                        x = xjscal_win.sel_js_num;
                        /* Reset the joystick records before caliberating. */
                        jscal_rec[x].x_min = MAX_JS_X_AXIS_VALUE;
                        jscal_rec[x].x_max = MIN_JS_X_AXIS_VALUE;
                        jscal_rec[x].y_min = MAX_JS_Y_AXIS_VALUE;
                        jscal_rec[x].y_max = MIN_JS_Y_AXIS_VALUE;

                        xjscal_win.mode_switch_state = MODE_SWITCH_CALIB;
                        XJSCalDrawSwitches();
                    }
                    /* Comma; decrease null zone. */
                    else if(event.xkey.keycode == osw_keycode.comma)
                    {
			x = xjscal_win.sel_js_num;
			jscal_rec[x].nullzone -= 1;

			if(jscal_rec[x].nullzone < 0)
			   jscal_rec[x].nullzone = 0;
                    }
                    /* Period; Increase null zone. */
                    else if(event.xkey.keycode == osw_keycode.period)
                    {
                        x = xjscal_win.sel_js_num;
                        jscal_rec[x].nullzone += 1;
		    }
		    break;

                  /* ************************************************** */
                  case KeyRelease:
                    /* Do not handle if not in focus. */
                    if(!xjscal_win.focused)
                        break;

		    /* Switch back on keyboard repeat. */
		    OSWKBAutoRepeatOn();

                    /* "X"; Exit. */
                    if(event.xkey.keycode == osw_keycode.alpha_x)
                    {
                        xjscal_win.exit_btn_state = False;
                        XJSCalDrawButtons();

                        runlevel = 1;
                    }
                    /* "S"; Save to file. */
                    else if(event.xkey.keycode == osw_keycode.alpha_s)
                    {
                        xjscal_win.save_btn_state = False;
                        XJSCalDrawButtons();

			XJSCalUnfocusAll();
			/* Map file browser. */
			FBrowserMapPath(&fb, fname.js_calib);
                    }
		    break;

                  /* ************************************************** */
		  case ButtonPress:
                    /* and is it the left pointer button? (1) */
                    if(event.xbutton.button == Button1)
                    {
                        if(event.xany.window == xjscal_win.save_btn)
			{
                            xjscal_win.save_btn_state = True;
                            XJSCalDrawButtons();
			}
                        else if(event.xany.window == xjscal_win.exit_btn)
			{
			    xjscal_win.exit_btn_state = True;
			    XJSCalDrawButtons();
			}
			else if(event.xany.window == xjscal_win.js_num_switch)
			{
			    /* Change switch only in testing mode. */
			    if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
			    	xjscal_win.sel_js_num++;

			    if(xjscal_win.sel_js_num >= MAX_JOYSTICKS)
				xjscal_win.sel_js_num = 0;

			    /* Close all joysticks. */
			    JSClose(&jscal_rec[0]);
                            JSClose(&jscal_rec[1]);
                            JSClose(&jscal_rec[2]);
                            JSClose(&jscal_rec[3]);

			    switch(xjscal_win.sel_js_num)
			    {
			      case 3:
                                JSOpen(DEV_JOYSTICK_3, &jscal_rec[3]);
                                strncpy(
                                    fname.js_device,
                                    DEV_JOYSTICK_3,
                                    PATH_MAX + NAME_MAX
                                );
                                break;

                              case 2:
                                JSOpen(DEV_JOYSTICK_2, &jscal_rec[2]);
                                strncpy(
                                    fname.js_device,
                                    DEV_JOYSTICK_2,
                                    PATH_MAX + NAME_MAX
                                );
                                break;

                              case 1:
                                JSOpen(DEV_JOYSTICK_1, &jscal_rec[1]);
				strncpy(
				    fname.js_device,
				    DEV_JOYSTICK_1,
				    PATH_MAX + NAME_MAX
			        );
                                break;

                              default:
                                JSOpen(DEV_JOYSTICK_0, &jscal_rec[0]);
                                strncpy(
                                    fname.js_device,
                                    DEV_JOYSTICK_0,
                                    PATH_MAX + NAME_MAX
                                );
                                break;
			    }
			    fname.js_device[PATH_MAX + NAME_MAX - 1] = '\0';

			    XJSCalDrawSwitches();
			}
                        else if(event.xany.window == xjscal_win.mode_switch)
			{
			    if(xjscal_win.mode_switch_state == MODE_SWITCH_TESTING)
			    {
                                x = xjscal_win.sel_js_num;  
                                /* Reset the joystick records before caliberating. */
                                jscal_rec[x].x_min = MAX_JS_X_AXIS_VALUE;
                                jscal_rec[x].x_max = MIN_JS_X_AXIS_VALUE;
                                jscal_rec[x].y_min = MAX_JS_Y_AXIS_VALUE;
                                jscal_rec[x].y_max = MIN_JS_Y_AXIS_VALUE;
                        
                                xjscal_win.mode_switch_state = MODE_SWITCH_CALIB;
			    }
			    else
                            {
                                xjscal_win.mode_switch_state = MODE_SWITCH_TESTING;
                            }

			    XJSCalDrawSwitches();
			}
		    }
		    break;

                  /* ************************************************** */
		  case ButtonRelease:
                    /* and is it the left pointer button? (1) */
                    if(event.xbutton.button == Button1)
                    {
                        if(event.xany.window == xjscal_win.save_btn)
			{
                            xjscal_win.save_btn_state = False;
                            XJSCalDrawButtons();

			    XJSCalUnfocusAll();
			    /* Map file browser for saving. */
                            FBrowserMapPath(&fb, fname.js_calib);
                    	}
                        else if(event.xany.window == xjscal_win.exit_btn)
			{
                            xjscal_win.exit_btn_state = False;
			    XJSCalDrawButtons();

			    runlevel = 1;
			}
		    }
		    break;

                  /* ************************************************** */
                  case Expose:
                    if((event.xany.window == xjscal_win.toplevel)
                    )      
                    {
                        XJSCalDrawAll();
                    }
                    break;

                  /* ************************************************** */
                  case FocusIn:
                    if(event.xany.window == xjscal_win.toplevel)
                    {
                        xjscal_win.focused = True;
                    }
		    else
		    {
			/* Use one toplevel window focus at a time rule. */
			xjscal_win.focused = False;
		    }
                    break;

                  /* ************************************************** */
                  case FocusOut:
                    if(event.xany.window == xjscal_win.toplevel)
                    {
                        xjscal_win.focused = False;
                    }
                    break;
		}


                /* Was the event from the GUI to delete toplevel? */
                if(OSWIsEventDestroyWindow(xjscal_win.toplevel, &event))
                {
                    runlevel = 1;
		}


		/* Dialog window. */
		DialogWinManage(&dialog, &event);

		/* File browser manage. */
		FBrowserManage(&fb, &event);

	    }	/* Managing GUI events. */


	    /* **************************************************** */
	    /* Redraw coordinates and button readouts. */

	    if(next.redraw_buttons < cur_millitime)
	    {
		XJSCalDrawReadout();
		next.redraw_buttons = cur_millitime + 100;
	    }

	    if(next.redraw_coordinates < cur_millitime)
	    {
                XJSCalDrawCoords();
		next.redraw_coordinates = cur_millitime + 100;
	    }


	    /* *************************************************** */
	    /* Do caliberation when in caliberation mode. */
	    if(xjscal_win.mode_switch_state == MODE_SWITCH_CALIB)
		XJSCalDoCalib();


	    /* Sleep */
	    usleep(PROG_LOOP_INTERVAL);
	}

	/* Free all resources and disconnect from GUI. */
        XJSCalShutdown();


	return(0);
}
